# -*- coding: utf8 -*-

from FGAme.events import EventDispatcher, signal


class Input(EventDispatcher):

    '''Objetos do tipo listener escutam eventos de entrada do usuário e executam
    os callbacks de resposta registrados a estes eventos.

    Esta classe define uma interface genérica que não implementa o método
    poll() que busca por eventos a cada frame de simulação. Este método deve
    ser implementado pelo backend específico (ex.: pygame, SDL2, etc)'''

    _instance = None
    long_press = signal('long-press', 'key')
    key_up = signal('key-up', 'key')
    key_down = signal('key-down', 'key')
    mouse_motion = signal('mouse-motion', num_args=1)
    mouse_button_up = signal('mouse-button-up', 'button', num_args=1)
    mouse_button_down = signal('mouse-button-down', 'button', num_args=1)
    mouse_long_press = signal('mouse-long-press', 'button', num_args=1)

    def __init__(self):
        super(Input, self).__init__()
        self._longpress_keys = set()
        self._longpress_mouse_buttons = set()
        self._mouse_pos = (0, 0)

        if self._instance is None:
            Input._instance = self
        else:
            raise RuntimeError('Input() is a singleton')

    # Callbacks globais para cada tipo de evento -----------------------------
    def process_key_down(self, key):
        '''Executa todos os callbacks associados à tecla fornecida'''

        self.trigger_key_down(key)
        self._longpress_keys.add(key)

    def process_key_up(self, key):
        '''Executa todos os callbacks de keyup associados à tecla fornecida'''

        self.trigger_key_up(key)
        self._longpress_keys.discard(key)

    def process_long_press(self):
        '''Executa todos os callbacks de longpress para as teclas
        pressionadas'''

        callback = self.trigger_long_press
        for key in self._longpress_keys:
            callback(key)

    def process_mouse_motion(self, pos):
        '''Executa callbacks associados ao movimento do mouse'''

        self.trigger_mouse_motion(pos)
        self._mouse_pos = pos

    def process_mouse_button_down(self, button, pos):
        '''Executa callbacks associados a pressionar um botão do mouse'''

        self.trigger_mouse_button_down(button, pos)
        self._longpress_mouse_buttons.add(button)

    def process_mouse_button_up(self, button, pos):
        '''Executa callbacks associados a levantar um botão previamente
        pressionado do mouse'''

        self.trigger_mouse_button_up(button, pos)
        self._longpress_mouse_buttons.discard(button)

    def process_mouse_longpress(self):
        '''Executa callbacks de long-press para os botões do mouse.

        A posição do ponteiro é guardada automaticamente no processamento dos
        sinais de 'mouse-motion'.'''

        pos = self._mouse_pos
        callback = self.trigger_mouse_long_press
        for button in self._longpress_mouse_buttons:
            callback(button, pos)

    #
    # Consulta as entradas de comandos pelo usuário
    #
    def poll(self):
        '''Função executada a cada loop, que investiga todos os eventos de
        usuários que ocorreram. Deve ser reimplementada nas classes filho'''

        raise NotImplementedError

    #
    # Funções utilizadas como funções globais no módulo FGAme.input
    #
    def _on_key_down(self, key, func=None, *args, **kwds):
        '''Registra função func para ser executada no frame em que a tecla
        fornecida for pressionada'''

        return self.listen_key_down(key, func, *args, **kwds)

    def _on_key_up(self, key, func=None, *args, **kwds):
        '''Registra função func para ser executada no frame em que o usuário
        liberar a tecla fornecida previamente pressionada'''

        return self.listen_key_up(key, func, *args, **kwds)

    def _on_long_press(self, key, func=None, *args, **kwds):
        '''Registra função func para ser executada em todos os frames em que a
        tecla fornecida estiver pressionada.'''

        return self.listen_long_press(key, func, *args, **kwds)

    def _on_mouse_motion(self, func=None, *args, **kwds):
        '''Registra função que é executada a cada frame em que o mouse se
        mover dentro da janela do jogo.

        A função recebe as coordenadas do ponteiro como primeiro argumento'''

        return self.listen_mouse_motion(func, *args, **kwds)

    def _on_mouse_button_down(self, button, func=None, *args, **kwds):
        '''Semelhante à `on_key_down()`, mas rastreia os cliques do mouse.

        A função de callback recebe a posição do ponteiro como primeiro
        argumento. Os botões são "left", "right", "middle", "wheel-up" e
        "wheel-down".'''

        return self.listen_mouse_button_down(func, *args, **kwds)

    def _on_mouse_button_up(self, button, func=None, *args, **kwds):
        '''Semelhante à `on_key_up()`, mas rastreia os cliques do mouse.

        A função de callback recebe a posição do ponteiro como primeiro
        argumento. Os botões são "left", "right", "middle", "wheel-up" e
        "wheel-down".'''

        return self.listen_mouse_button_up(func, *args, **kwds)

    def _on_mouse_long_press(self, button, func=None, *args, **kwds):
        '''Semelhante à `on_long_press()`, mas rastreia os cliques do mouse.

        A função de callback recebe a posição do ponteiro como primeiro
        argumento. Os botões são "left", "right", "middle", "wheel-up" e
        "wheel-down".'''

        return self.listen_mouse_button_long_press(func, *args, **kwds)


#
# Registra as funções globais
#
def _register(func_name):
    input_method = getattr(Input, func_name)

    def signal_handler_func(*args, **kwds):
        try:
            worker = getattr(Input._instance, func_name)
        except AttributeError:
            raise RuntimeError(
                'Input system has not started. '
                'Have you tried to execute conf.init() first?')
        return worker(*args, **kwds)

    signal_handler_func.__name__ = input_method.__name__
    signal_handler_func.__doc__ = input_method.__doc__
    return signal_handler_func


on_key_down = _register('_on_key_down')
on_key_up = _register('_on_key_up')
on_long_press = _register('_on_long_press')
on_mouse_motion = _register('_on_mouse_motion')
on_mouse_button_down = _register('_on_mouse_button_down')
on_mouse_button_up = _register('_on_mouse_button_up')
on_mouse_long_press = _register('_on_mouse_long_press')



from math import trunc
import ctypes
import sdl
import sdl.sdlgfx as gfx
from FGAme.core.screen import Canvas
from FGAme.input import Input
from FGAme.mainloop import MainLoop
from FGAme.draw import Color

#
# Module constants
#
HAS_INIT_SDL_VIDEO = False


class SDL2Canvas(Canvas):

    '''Implementa a interface Screen, utilizando a biblioteca Pygame'''

    # Local class-based caches
    _gfx = gfx
    _sdl = sdl

    def show(self):
        # Init video
        global HAS_INIT_SDL_VIDEO
        if not HAS_INIT_SDL_VIDEO:
            self._no_error(sdl.SDL_Init(sdl.SDL_INIT_VIDEO))
        else:
            HAS_INIT_SDL_VIDEO = True

        # Create window
        self._window = self._no_null(
            sdl.SDL_CreateWindow(
                b"FGAme App",
                sdl.SDL_WINDOWPOS_CENTERED,
                sdl.SDL_WINDOWPOS_CENTERED,
                self.width,
                self.height,
                sdl.SDL_WINDOW_SHOWN))

        # Create a renderer
        flags = sdl.SDL_RENDERER_SOFTWARE
        self._renderer = self._no_null(
            sdl.SDL_CreateRenderer(self._window, -1, flags))

        # Create a rect instance that stores the window shape
        self._screen_rect = self._no_null(
            sdl.SDL_Rect(x=0, y=0, w=self.width, h=self.height))

        # Saves background color
        if self.background is not None:
            R, B, G, A = Color(self.background)
            self._bg_color = (R << 24) + (G << 16) + (B << 8) + A
        else:
            self._bg_color = None
        self._LP_short = gfx.aapolygonRGBA.argtypes[1]

    def flip(self):
        sdl.SDL_RenderPresent(self._renderer)

    def _map_point(self, point):
        x, y = point
        return (trunc(x), trunc(self.height - y))

    def _sdl_error(self):
        msg = sdl.SDL_GetError()
        raise RuntimeError('SDL error: %s' % msg)

    def _no_error(self, value):
        if value != 0:
            msg = sdl.SDL_GetError()
            raise RuntimeError('SDL error (%s): %s' % (value, msg))

    def _no_null(self, value):
        if not value:
            msg = sdl.SDL_GetError()
            raise RuntimeError('SDL error (%s): %s' % (value, msg))
        return value

    #
    # Desenho de figuras geométricas primitivas
    #
    def draw_raw_aabb_solid(self, aabb, color):
        Y = self.height
        R, G, B, A = color
        if gfx.boxRGBA(
                self._renderer,
                trunc(aabb.xmin),
                trunc(Y - aabb.ymax),
                trunc(aabb.xmax),
                trunc(Y - aabb.ymin),
                R, G, B, A) != 0:
            self._sdl_error()

    def draw_raw_circle_solid(self, circle, color):
        self._no_error(
            gfx.filledCircleRGBA(
                self._renderer,
                trunc(circle.x),
                trunc(self.height - circle.y),
                trunc(circle.radius), *color))

        self._no_error(
            gfx.aacircleRGBA(
                self._renderer,
                trunc(circle.x),
                trunc(self.height - circle.y),
                trunc(circle.radius), *color))

    def draw_raw_circle_border(self, circle, width, color):
        safe_operator = self._no_error
        x, y = trunc(circle.x), trunc(self.height - circle.y)
        renderer = self._renderer

        if width == 1:
            radius = trunc(circle.radius)
            safe_operator(gfx.aacircleRGBA(renderer, x, y, radius, *color))
        else:
            # Very ugly hack! can it be better?
            min_r = trunc(circle.radius - width / 2)
            max_r = trunc(circle.radius + width / 2)
            for r in range(min_r, max_r):
                safe_operator(gfx.aacircleRGBA(renderer, x, y, r, *color))

    def _get_poly_xy(self, poly):
        '''Implementação comum que cria lista de posições x e y para passsar
        para draw_raw_poly_solid e draw_raw_poly_border'''
        N = len(poly)
        height = self.height
        int16_array = ctypes.c_int16 * N
        Xc = int16_array()
        Yc = int16_array()
        for i, pt in enumerate(poly):
            Xc[i] = trunc(pt.x)
            Yc[i] = trunc(height - pt.y)
        return Xc, Yc

    def draw_raw_poly_solid(self, poly, color):
        N = len(poly)
        X, Y = self._get_poly_xy(poly)
        color = Color(color)
        self._no_error(
            gfx.aapolygonRGBA(
                self._renderer, X, Y, N, *color))
        self._no_error(
            gfx.filledPolygonRGBA(
                self._renderer, X, Y, N, *color))

    def draw_raw_segment(self, segment, width, color):
        height = self.height
        pt1, pt2 = segment
        if width == 1:
            self._no_error(
                gfx.aalineRGBA(
                    self._renderer,
                    trunc(pt1.x),
                    trunc(height - pt1.y),
                    trunc(pt2.x),
                    trunc(height - pt2.y),
                    *color))
        else:
            self._no_error(
                gfx.thickLineRGBA(
                    self._renderer,
                    trunc(pt1.x),
                    trunc(height - pt1.y),
                    trunc(pt2.x),
                    trunc(height - pt2.y),
                    trunc(width),
                    *color))

    def draw_raw_poly_border(self, poly, width, color):
        # TODO: desenhar linhas espessas
        N = len(poly)
        X, Y = self._get_poly_xy(poly)
        self._no_error(
            gfx.aapolygonRGBA(
                self._renderer, X, Y, N, *Color(color)))

    def clear_background(self, color=None):
        renderer = self._renderer
        sdl.SDL_RenderClear(renderer)

        ret = 0
        if color is None and self._bg_color is not None:
            ret = gfx.boxColor(
                renderer, 0, self.height, self.width, 0, self._bg_color)
        else:
            R, G, B, A = Color(color)
            ret = gfx.boxRGBA(
                renderer, 0, self.height, self.width, 0, R, G, B, A)

        if ret != 0:
            msg = sdl.SDL_GetError()
            raise RuntimeError('SDL error: %s' % msg)


class SDL2Input(Input):

    '''Objetos do tipo listener.'''

    #=========================================================================
    # Conversões entre strings e teclas
    #=========================================================================
# Setas direcionais
#     KEY_CONVERSIONS = {
#         'up': SDLK_UP, 'down': SDLK_DOWN, 'left': SDLK_LEFT, 'right': SDLK_RIGHT,
#         'return': SDLK_RETURN, 'space': SDLK_SPACE, 'enter': SDLK_RETURN,
#     }
#
# Adiciona as letras e números
#     chars = '0123456789' + string.ascii_lowercase
#     for c in chars:
#         KEY_CONVERSIONS[c] = getattr(sdl, 'SDLK_' + c)

    #
    # Laço principal de escuta de eventos
    #

    def poll(self):
        for event in sdl_ext.get_events():
            if event.type == sdl.SDL_QUIT:
                raise SystemExit
            elif event.type == sdl.SDL_KEYDOWN:
                self.on_key_down(event.key.keysym.sym)
            elif event.type == sdl.SDL_KEYUP:
                self.on_key_up(event.key.keysym.sym)
            elif event.type == sdl.SDL_MOUSEMOTION:
                # TODO: converter para coordenadas locais em screen
                # self.on_mouse_motion(event.pos)
                pass

        # self.on_long_press()


class SDL2MainLoop(MainLoop):

    '''The SDL2 main loop'''



from FGAme import conf
from FGAme import draw

screen = conf.init_screen(800, 600)
assert screen is conf.get_screen()
screen.show()

circle = draw.Circle(100, color='red')
screen.draw(circle)
screen.flip()


c1 = draw.Circle(50, pos=(400, 300), color='black')
c2 = draw.Circle(30, pos=(400, 300), color='white')
with screen.painting():
    screen.draw(c1)
    screen.draw(c2)


# -*- coding: utf8 -*-

import string
import pygame
from FGAme import conf
from FGAme.core import Canvas
from FGAme.input import Input
from FGAme.mainloop import MainLoop
from FGAme.draw import Color, rgb
from FGAme.util import autodoc
pygame.init()  # @UndefinedVariable

black = Color('black')
white = Color('white')


@autodoc
class PyGameCanvas(Canvas):

    '''Implementa a interface Canvas utilizando a biblioteca pygame'''

    _pg_draw_circle = pygame.draw.circle
    _pg_draw_rect = pygame.draw.rect
    _pg_draw_poly = pygame.draw.polygon
    _pg_segment = pygame.draw.line

    def show(self):
        self._screen = pygame.display.set_mode((self.width, self.height))
        super(PyGameCanvas, self).show()

    def get_pygame_screen(self):
        return self._screen

    def flip(self):
        pygame.display.update()

    #
    # Funções de desenho - chama as funções de pygame.draw
    #
    def draw_raw_aabb_solid(self, aabb, color=black):
        self.draw_raw_aabb_border(aabb, 0, color)

    def draw_raw_aabb_border(self, aabb, width=1.0, color=black):
        Y = self.height
        x, x_, y, y_ = map(int, aabb)
        rect = (x, Y - y_, x_ - x, y_ - y)
        self._pg_draw_rect(
            self._screen,
            color,       # color
            rect,        # rect
            int(width))  # line width

    def draw_raw_circle_solid(self, circle, color=black):
        self.draw_raw_circle_border(circle, 0, color)

    def draw_raw_circle_border(self, circle, width=1.0, color=black):
        Y = self.height
        x, y = circle.pos
        self._pg_draw_circle(
            self._screen,
            color,                 # line color
            (int(x), Y - int(y)),  # center
            int(circle.radius),    # radius
            int(width))            # line width

    def draw_raw_poly_solid(self, poly, color=black):
        self.draw_raw_poly_border(poly, 0, color)

    def draw_raw_poly_border(self, poly, width=1.0, color=black):
        Y = self.height
        vertices = [(int(x), int(Y - y)) for (x, y) in poly]
        self._pg_draw_poly(
            self._screen,
            color,        # line color
            vertices,     # center
            int(width))   # line width

    def draw_raw_segment(self, segment, width=1.0, color=black):
        Y = self.height
        pt1, pt2 = [(int(x), int(Y - y)) for (x, y) in segment]
        self._pg_segment(self._screen, color, pt1, pt2, int(width))

    def draw_raw_texture(self, texture, pos=(0, 0)):
        try:
            pg_texture = texture.data
        except AttributeError:
            pil_data = texture.get_pil_data()
            if pil_data.mode not in 'RGB' or 'RGBA':
                pil_data = pil_data.convert('RGBA')
            pg_texture = pygame.image.fromstring(
                pil_data.tostring(),
                texture.shape, pil_data.mode)
            texture.set_data(pg_texture)

        x, y, dx, dy = pg_texture.get_rect()
        x += pos[0]
        y += pos[1] + dy
        self._screen.blit(pg_texture, (x, self.height - y, dx, dy))

    def paint_pixel(self, pos, color=Color(0, 0, 0)):
        x, y = self._map_point(*pos)
        # TODO: talvez use pygame.display.get_surface() para obter a tela
        # correta
        # http://stackoverflow.com/questions/10354638/pygame-draw-single-pixel
        self._screen.set_at(x, y, rgb(color))

    def clear_background(self, color=None):
        if color is None:
            if self.background is None:
                raise RuntimeError('background was not defined')
            self._screen.fill(self.background)
        else:
            self._screen.fill(Color(color))


class PyGameInput(Input):

    '''Implementa a interface Input através do Pygame.'''

    def __init__(self):
        super(PyGameInput, self).__init__()
        _pg = pygame
        D = dict(
            up=_pg.K_UP,
            down=_pg.K_DOWN,
            left=_pg.K_LEFT,
            right=_pg.K_RIGHT,
            space=_pg.K_SPACE,
        )
        D['return'] = _pg.K_RETURN

        # Adiciona as letras e números
        for c in '0123456789' + string.ascii_lowercase:
            D[c] = getattr(_pg, 'K_' + c)

        self._key_conversions = {v: k for (k, v) in D.items()}
        self._mouse_conversions = {
            1: 'left',
            2: 'middle',
            3: 'right',
            4: 'wheel-up',
            5: 'wheel-down'
        }

    # Laço principal de escuta de eventos #####################################
    def poll(self):
        key_get = self._key_conversions.get
        mouse_button_get = self._mouse_conversions.get
        window_height = conf.get_resolution()[1]
        pg = pygame

        for event in pygame.event.get():
            if event.type == pg.QUIT:
                raise SystemExit
            elif event.type == pg.KEYDOWN:
                self.process_key_down(key_get(event.key))
            elif event.type == pg.KEYUP:
                self.process_key_up(key_get(event.key))
            elif event.type == pg.MOUSEMOTION:
                x, y = event.pos
                y = window_height - y
                self.process_mouse_motion((x, y))
            elif event.type == pg.MOUSEBUTTONUP:
                x, y = event.pos
                y = window_height - y
                button = mouse_button_get(event.button)
                self.process_mouse_button_up(button, (x, y))
            elif event.type == pg.MOUSEBUTTONDOWN:
                x, y = event.pos
                y = window_height - y
                button = mouse_button_get(event.button)
                self.process_mouse_button_down(button, (x, y))

        self.process_long_press()
        self.process_mouse_longpress()


class PyGameMainLoop(MainLoop):
    pass



from __future__ import division
from FGAme.backends.pygame import PyGameCanvas

class PyGameGLCanvas(PyGameCanvas):
    '''Implementa a interface Screen, utilizando a biblioteca Pygame'''

    def __init__(self, w, h, *args, **kwds):
        kwds['set_mode_args'] = [HWSURFACE|OPENGL|DOUBLEBUF]
        super(PyGameGLScreen, self).__init__(w, h, *args, **kwds)

    def show(self):
        pygame.display.flip()

    def draw_circle(self, pos, radius, color=(0, 0, 0), solid=True):
        x, y = self._map_point(pos)
        #pygame.gfxdraw.aacircle(self._screen, x, y, trunc(radius), color)
        #if solid:
        #    pygame.gfxdraw.filled_circle(self._screen, x, y, trunc(radius), color)

    def draw_poly(self, points, color=(0, 0, 0), solid=True):
        points = [ self._map_point(pt) for pt in points ]
        #pygame.gfxdraw.aapolygon(self._screen, points, color)
        #if solid:
        #    pygame.gfxdraw.filled_polygon(self._screen, points, color)

    def draw_rect(self, pos, shape, color=(0, 0, 0), solid=True):
        Screen.draw_rect(self, pos, shape, color, solid)

    def clear(self, color=None):
        R, G, B = color or self.background
        glClearColor(R/255., G/255., B/255., 0.0)
        


'''
Implementa as interfaces da classe Body. Evita uma implementação monolítica
de uma classe enorme e favorece a reutilização e modularização do código.
'''

from FGAme.mathtools import Vec2, sqrt, null2D
from FGAme.util import popattr

INF = float('inf')


class HasAABB(object):

    '''
    Objetos que possuem uma AABB definida no atributo ``obj.aabb`` e uma
    posição definida no attributo ``obj.pos``.

    O centro da AABB não precisa coincidir com o valor de ``obj.pos``. Neste
    caso, as referências, pos_up, pos_down, pos_left e pos_right utilizam as
    coordenadas de ``obj.pos`` como referências do centro.
    '''

    __slots__ = []

    @property
    def xmin(self):
        return self.aabb.xmin

    @property
    def xmax(self):
        return self.aabb.xmax

    @property
    def ymin(self):
        return self.aabb.ymin

    @property
    def ymax(self):
        return self.aabb.ymax

    @property
    def bbox(self):
        return (self.xmin, self.xmax, self.ymin, self.ymax)

    @property
    def shape(self):
        return (self.xmax - self.xmin, self.ymax - self.ymin)

    @property
    def width(self):
        return self.xmax - self.xmin

    @property
    def height(self):
        return self.ymax - self.ymin

    @property
    def rect(self):
        x, y = self.xmin, self.ymin
        return (x, y, self.xmax - x, self.ymax - y)

    @property
    def pos_sw(self):
        return Vec2(self.xmin, self.ymin)

    @property
    def pos_se(self):
        return Vec2(self.xmax, self.ymin)

    @property
    def pos_nw(self):
        return Vec2(self.xmin, self.ymax)

    @property
    def pos_ne(self):
        return Vec2(self.xmax, self.ymax)

    @property
    def pos_right(self):
        return Vec2(self.xmax, self.pos.y)

    @property
    def pos_left(self):
        return Vec2(self.xmin, self.pos.y)

    @property
    def pos_up(self):
        return Vec2(self.pos.x, self.ymax)

    @property
    def pos_down(self):
        return Vec2(self.pos.x, self.ymin)


class HasInertia(object):

    '''Define um objeto que possui propriedades de inércia linear (massa) e
    angular (momento de inércia).'''

    __slots__ = []

    @property
    def invmass(self):
        return self._invmass

    @property
    def invinertia(self):
        return self._invinertia

    @property
    def mass(self):
        try:
            return 1.0 / self._invmass
        except ZeroDivisionError:
            return float('inf')

    @mass.setter
    def mass(self, value):
        value = float(value)

        if value <= 0:
            raise ValueError('mass cannot be null or negative')
        elif value != INF:
            self._density = value / self.area()
            if self._invinertia:
                inertia = value * self.ROG_sqr()
                self._invinertia = 1.0 / inertia
            self._invmass = 1.0 / value
        else:
            self._invmass = 0.0
            self._invinertia = 0.0

    @property
    def inertia(self):
        try:
            return 1.0 / self._invinertia
        except ZeroDivisionError:
            return INF

    @inertia.setter
    def inertia(self, value):
        value = float(value)

        if self.can_rotate:
            if value <= 0:
                raise ValueError('inertia cannot be null or negative')
            elif value != INF:
                self._invinertia = 1.0 / value
            else:
                self._invinertia = 0.0
        else:
            self._raise_cannot_rotate_error()

    @property
    def density(self):
        return self._density

    @density.setter
    def density(self, value):
        rho = float(value)
        self._density = rho
        if self._invmass:
            self._invmass = 1.0 / (self.area() * rho)
        if self._invinertia:
            self._invinertia = 1.0 / (self.area() * rho * self.ROG_sqr())

    #
    # Propriedades Geométricas
    #
    def area(self):
        '''Área do objeto'''

        try:
            return self._baseshape.area()
        except AttributeError:
            return 0.0

    def ROG_sqr(self):
        '''Raio de giração ao quadrado.'''

        try:
            return self._baseshape.ROG_sqr()
        except AttributeError:
            return float('inf')

    def ROG(self):
        '''Raio de giração.

        Subclasses devem sobrescrever ROG_sqr, ao invés deste método.'''

        return sqrt(self.ROG_sqr())

    ###########################################################################
    #               Manipulação/consulta do estado dinâmico
    ###########################################################################
    # Simplificar e usar apenas a massa e a flag can_rotate como quesito de
    # teste.
    # FIXME: Alterar as flags nas funções make_*
    def is_dynamic(self, what=None):
        '''Retorna True se o objeto for dinâmico ou nas variáveis lineares ou
        nas angulares. Um objeto é considerado dinâmico nas variáveis lineares
        se possuir massa finita. De maneira análoga, o objeto torna-se dinâmico
        nas variáveis angulares se possuir momento de inércia finito.

        Opcionalmente pode especificar um parâmetro posicional 'linear',
        'angular', 'both' or 'any' (padrão) para determinar o tipo de consulta
        a ser realizada'''

        if what is None or what == 'any':
            return self.is_dynamic_linear() or self.is_dynamic_angular()
        elif what == 'both':
            return self.is_dynamic_linear() and self.is_dynamic_angular()
        elif what == 'linear':
            return self.is_dynamic_linear()
        elif what == 'angular':
            return self.is_dynamic_angular()
        else:
            raise ValueError('unknown mode: %r' % what)

    def is_dynamic_linear(self):
        '''Verifica se o objeto é dinâmico nas variáveis lineares'''

        return bool(self._invmass)

    def is_dynamic_angular(self):
        '''Verifica se o objeto é dinâmico nas variáveis angulares'''

        return bool(self._invinertia)

    def make_dynamic(self, what=None, restore_speed=True):
        '''Resgata a massa, inércia e velocidades anteriores de um objeto
        paralizado pelo método `obj.make_static()` ou `obj.make_kinematic()`.

        Aceita um argumento com a mesma semântica de is_dynamic()
        '''

        if what is None or what == 'both':
            self.make_dynamic_linear(restore_speed)
            self.make_dynamic_angular(restore_speed)
        elif what == 'linear':
            self.make_dynamic_linear(restore_speed)
        elif what == 'angular':
            self.make_dynamic_angular(restore_speed)
        else:
            raise ValueError('unknown mode: %r' % what)

    def make_dynamic_linear(self, restore_speed=True):
        '''Resgata os parâmetros dinâmicos lineares de um objeto estático ou
        cinemático paralizado pelos métodos `obj.make_static()` ou
        `obj.make_kinematic()`.'''

        if not self.is_dynamic_linear():
            self._invmass = 1.0 / (self.area() * self._density)

            # Resgata a velocidade
            if restore_speed and self._vel.is_null():
                self._vel = popattr(self, '_old_vel', null2D)

    def make_dynamic_angular(self, restore_speed=True):
        '''Resgata os parâmetros dinâmicos angulares de um objeto estático ou
        cinemático paralizado pelos métodos `obj.make_static()` ou
        `obj.make_kinematic()`.'''

        if not self.is_dynamic_angular():
            self._inertia = 1.0 / (self._density * self.ROG_sqr())

            # Resgata a velocidade
            if restore_speed and self._omega == 0:
                self._omega = popattr(self, '_old_omega', 0.0)

    # Kinematic ###############################################################
    def is_kinematic(self, what=None):
        '''Retorna True se o objeto for cinemático ou nas variáveis lineares ou
        nas angulares. Um objeto é considerado cinemático (em uma das
        variáveis) se não for dinâmico. Se, além de cinemático, o objeto
        possuir velocidade nula, ele é considerado estático.

        Opcionalmente pode especificar um parâmetro posicional 'linear',
        'angular', 'both' (padrão) or 'any' para determinar o tipo de consulta
        a ser realizada.
        '''

        if what is None or what == 'both':
            return not (self.is_dynamic_linear() or self.is_dynamic_angular())
        elif what == 'any':
            return (
                not self.is_dynamic_linear()) or (
                not self.is_dynamic_angular())
        elif what == 'linear':
            return not self.is_dynamic_linear()
        elif what == 'angular':
            return not self.is_dynamic_angular()
        else:
            raise ValueError('unknown mode: %r' % what)

    def is_kinematic_linear(self):
        '''Verifica se o objeto é dinâmico nas variáveis lineares'''

        return not self.is_dynamic_linear()

    def is_kinematic_angular(self):
        '''Verifica se o objeto é dinâmico nas variáveis angulares'''

        return not self.is_dynamic_angular()

    def make_kinematic(self, what=None):
        '''Define a massa e/ou inércia como infinito.

        Aceita um argumento com a mesma semântica de is_kinematic()
        '''

        if what is None or what == 'both':
            self.make_kinematic_linear()
            self.make_kinematic_angular()
        elif what == 'linear':
            self.make_kinematic_linear()
        elif what == 'angular':
            self.make_kinematic_angular()
        else:
            raise ValueError('unknown mode: %r' % what)

    def make_kinematic_linear(self):
        '''Resgata os parâmetros dinâmicos lineares de um objeto estático ou
        cinemático paralizado pelos métodos `obj.make_static()` ou
        `obj.make_kinematic()`.'''

        self._invmass = 0.0

    def make_kinematic_angular(self):
        '''Resgata os parâmetros dinâmicos angulares de um objeto estático ou
        cinemático paralizado pelos métodos `obj.make_static()` ou
        `obj.make_kinematic()`.'''

        self._invinertia = 0.0

    # Static ##################################################################
    def is_static(self, what=None):
        '''Retorna True se o objeto for estatático nas variáveis lineares e
        nas angulares. Um objeto é considerado estático (em uma das variáveis)
        se além de cinemático, a velocidade for nula.

        Opcionalmente pode especificar um parâmetro posicional 'linear',
        'angular', 'both' (padrão) or 'any' para determinar o tipo de consulta
        a ser realizada'''

        if what is None or what == 'both':
            return self.is_static_linear() and self.is_static_angular()
        elif what == 'any':
            return self.is_static_linear() or self.is_static_angular()
        elif what == 'linear':
            return self.is_static_linear()
        elif what == 'angular':
            return self.is_static_angular()
        else:
            raise ValueError('unknown mode: %r' % what)

    def is_static_linear(self):
        '''Verifica se o objeto é dinâmico nas variáveis lineares'''

        return self.is_kinematic_linear() and self._vel == null2D

    def is_static_angular(self):
        '''Verifica se o objeto é dinâmico nas variáveis angulares'''

        return self.is_kinematic_angular() and self._omega == 0

    def make_static(self, what=None):
        '''Define a massa e/ou inércia como infinito.

        Aceita um argumento com a mesma semântica de is_static()
        '''

        if what is None or what == 'both':
            self.make_static_linear()
            self.make_static_angular()
        elif what == 'linear':
            self.make_static_linear()
        elif what == 'angular':
            self.make_static_angular()
        else:
            raise ValueError('unknown mode: %r' % what)

    def make_static_linear(self):
        '''Resgata os parâmetros dinâmicos lineares de um objeto estático ou
        cinemático paralizado pelos métodos `obj.make_static()` ou
        `obj.make_kinematic()`.'''

        self.make_kinematic_linear()
        self._old_vel = self._vel
        self._vel = null2D

    def make_static_angular(self):
        '''Resgata os parâmetros dinâmicos angulares de um objeto estático ou
        cinemático paralizado pelos métodos `obj.make_static()` ou
        `obj.make_kinematic()`.'''

        self.make_kinematic_angular()
        self._old_omega = self._omega
        self._omega = 0.0

    def _raise_cannot_rotate_error(self):
        raise ValueError('Cannot change angular variables with disabled '
                         '`can_rotate` flag')


class HasGlobalForces(object):

    '''
    Define um objeto que pode definir forças globais como gravidade e
    amortecimento.
    '''
    __slots__ = []

    @property
    def gravity(self):
        return self._gravity

    @gravity.setter
    def gravity(self, value):
        try:
            self._gravity = Vec2(*value)
        except TypeError:
            self._gravity = Vec2(0, -value)
        self.owns_gravity = True

    @property
    def damping(self):
        return self._damping

    @damping.setter
    def damping(self, value):
        self._damping = float(value)
        self.owns_damping = True

    @property
    def adamping(self):
        return self._adamping

    @adamping.setter
    def adamping(self, value):
        self._adamping = float(value)
        self.owns_adamping = True

    @property
    def restitution(self):
        return self._restitution

    @restitution.setter
    def restitution(self, value):
        self._restitution = float(value)
        self.owns_restitution = True

    @property
    def friction(self):
        return self._friction

    @friction.setter
    def friction(self, value):
        self._friction = float(value)
        self.owns_friction = True



from FGAme import World, Circle, AABB, pos, conf, on_key_down
import random
conf.set_framerate(60)


world = World()
circle_list = []
for _ in range(5):
    circle = Circle(50, pos=pos.middle, world=world, color='random')
    circle_list.append(circle)

o1 = AABB(pos=pos.middle, shape=(80, 71))
o2 = AABB(pos=pos.middle, shape=(80, 71), image='alien1')
world.add(o1)
world.add(o2, layer=2)


@world.listen('frame-enter')
def frame_enter_handler():
    for circle in circle_list:
        x = random.random() * 800
        y = random.random() * 600
        circle.pos = x, y


@on_key_down('space')
def space():
    world.toggle_pause()

world.run()



# -*- coding: utf8 -*-

from __future__ import division
from math import sin, cos
from random import uniform
import kivy
from kivy.app import App
from kivy.lang import Builder
from kivy.clock import Clock
from kivy.properties import ListProperty
from kivy.uix.floatlayout import FloatLayout
from kivy.animation import Animation
from kivy.graphics import Color, Ellipse

kv = '''
<WorldLayout>:
    canvas:
        Color:
            rgba: 0, 0, 1, 1
        Ellipse:
            pos: self.pos_circle
            size: 60, 60

    FloatLayout:
        ToggleButton:
            text: 'Run'
            size_hint_y: 0.1
            size_hint_x: 0.1
            on_press: root.animate()
'''
Builder.load_string(kv)


class WorldLayout(FloatLayout):
    pos_circle = ListProperty([400 - 30, 300 - 30])
    state = ListProperty([0, 1, 1, 0])
    omega = 20

    def update_world(self, dt):
        x, vx, y, vy = self.state
        x += vx * dt
        vx -= self.omega * x * dt
        y += vy * dt
        vy -= self.omega * y * dt
        self.state = [x, vx, y, vy]
        self.pos_circle = [200 * x + 400, 300 + 200 * y]

    def animate(self):
        Clock.schedule_interval(self.update_world, 1. / 60)


class WorldLayout(FloatLayout):
    pos_circle = ListProperty([400 - 30, 300 - 30])
    state = ListProperty([0, 1, 1, 0])
    omega = 20

    def __init__(self):
        super(WorldLayout, self).__init__()
        with self.canvas:
            Color(0, 0, 1)
            self.circle = Ellipse(pos=(370, 270), size=(60, 60))

    def update_world(self, dt):
        x, vx, y, vy = self.state
        x += vx * dt
        vx -= self.omega * x * dt
        y += vy * dt
        vy -= self.omega * y * dt
        self.state = [x, vx, y, vy]
        anim = Animation(pos=(200 * x + 400, 300 + 200 * y), duration=dt)
        anim.start(self.circle)

    def animate(self):
        Clock.schedule_interval(self.update_world, 1. / 20)


class RotationApp(App):

    def build(self):
        return WorldLayout()

if __name__ == '__main__':
    RotationApp().run()



#-*- coding: utf8 -*-
from __future__ import print_function

from FGAme import *
from random import uniform, choice
from FGAme.effects import explode

print('Starting simulation...')
random_color = lambda: (255, 0, 0)
world = World()

obj1 = Poly.regular(N=8, length=100, pos=(-100, -100), vel=(100, 100),
                    omega=1, color=random_color())
obj1.mass = 100
world.add(obj1)

obj2 = Poly.regular(N=4, length=50, pos=(200, 200), vel=(-100, -100),
                    omega=-uniform(0, 4), color=random_color())
obj2.mass = 100
world.add(obj2)

aabb = AABB(bbox=(-100, 400, -300, -200), color='black', mass='inf')
world.add(aabb)

aabb2 = AABB(bbox=(300, 380, -150, 280), color='black', mass='inf')
world.add(aabb2)

obj3 = Poly.regular(N=5, length=150, pos=(-300, 200), vel=(-100, -100),
                   omega=-uniform(0, 4), color=random_color())
obj3.mass = 100
obj3.make_static()
world.add(obj3)

explode(obj1, world, energy=obj1.mass * 100 ** 2, prob_rec=1)

def pause():
    obj1.pause()
    obj2.pause()

def unpause():
    obj1.unpause()
    obj2.unpause()

# Inicia a simulação
world.run()



# -*- coding: utf8 -*-
'''
Implementação do tradicional jogo Pong.
'''

from FGAme import *
from random import uniform, choice, random


class Pong(World):

    def __init__(self, **kwds):
        super(Pong, self).__init__()
        self.add_bounds(width=(-300, 10))

        # Linha central
        self.add(draw.AABB(shape=(15, 550), pos=(400, 300),
                           color=(200, 200, 200)))

        # Cria a bola com uma velocidade aleatória
        self.ball = Circle(30, color='red', world=self)
        self.ball.pos = (100, 300)
        self.ball.vel = (+700, choice([-1, 1]) * uniform(200, 400))

        # Cria a barras
        self.pong1 = AABB(shape=[20, 130], pos=(50, 300),
                          world=self, mass='inf')
        self.pong2 = AABB(shape=[20, 130], pos=(750, 300),
                          world=self, mass='inf')

        # Registra eventos
        self.listen('long-press', 'up', self.move_up, obj=self.pong2)
        self.listen('long-press', 'down', self.move_down, obj=self.pong2)
        self.listen('long-press', 'w', self.move_up, obj=self.pong1)
        self.listen('long-press', 's', self.move_down, obj=self.pong1)
        self.listen('key-down', 'space', self.toggle_pause)

    def move_up(self, obj):
        '''Move a raquete fornecida para cima'''

        if obj.ymax < 590:
            obj.move(Vec2(0, 10))

    def move_down(self, obj):
        '''Move a raquete fornecida para baixo'''

        if obj.ymin > 10:
            obj.move(Vec2(0, -10))

if __name__ == '__main__':
    Pong().run()



# -*- coding: utf8 -*-
from __future__ import division
from __future__ import print_function

from FGAme.mathutils import *
from FGAme import *
from random import uniform, choice


def random_color():
    return tuple(int(uniform(0, 255)) for i in range(3))


class Scenario:

    def floor(self):
        floor = AABB(bbox=(-10000, 10000, 0, 150), color=(68, 170, 0))
        floor.make_static()
        return floor

    def pole(self):
        pole = AABB(bbox=(50, 60, 150, 350), color=(158, 86, 38))
        pole.make_static()
        return pole

    def boxes(self):
        height = 80
        width = 200
        pos = Vec2(650, 190)
        shape = (width, height)
        p = Rectangle(pos=pos, shape=shape, color=(158, 86, 38))
        yield p

        for _ in range(4):
            shapeX, shapeY = shape
            height *= 0.75
            width *= 2. / 3
            deltay = shapeY / 2. + height / 2. - 1
            deltax = uniform(-shapeX / 6., shapeX / 6.)
            pos = pos + (deltax, deltay)
            shape = (width, height)
            yield Rectangle(pos=pos, shape=shape, color=(158, 86, 38))

    def get_objects(self):
        yield self.floor()
        yield self.pole()
        for box in self.boxes():
            yield box


# Inicializa o cenário
scene = Scenario()

# Cria triângulo
L = 40
h = 10 * sqrt(12)
tri = RegularPoly(3, L, color=(200, 0, 0), density=50)
tri.move((65, 250 + L))
tri.boost((350, 350))
tri.aboost(-5)
# tri.is_dynamic_angular = False

# Inicializa o mundo
world = World(background=(0, 204, 255),
              gravity=500,
              dfriction=0.3,
              sfriction=0.5,
              restitution=0.7)
for obj in scene.get_objects():
    world.add(obj)
world.add(tri)

# Inicia a simulação
world.run()



# -*- coding: utf8 -*-
from itertools import count


class _BodyFlagsBase:
    N = count()

    # Estado dinâmico do objeto
    active = 1 << next(N)
    is_dynamic = 1 << next(N)
    is_kinematic = 1 << next(N)
    is_static = 1 << next(N)
    is_sensor = 1 << next(N)
    is_sleeping = 1 << next(N)
    can_rotate = 1 << next(N)
    can_sleep = 1 << next(N)

    # Controle de forças
    owns_gravity = 1 << next(N)
    owns_damping = 1 << next(N)
    owns_adamping = 1 << next(N)
    owns_restitution = 1 << next(N)
    owns_friction = 1 << next(N)

    # Contatos e vínculos
    has_joints = 1 << next(N)
    has_contacts = 1 << next(N)
    has_external_force = 1 << next(N)
    has_external_accel = 1 << next(N)
    has_external_torque = 1 << next(N)
    has_external_alpha = 1 << next(N)

    # Estados temporários
    dirty_shape = 1 << next(N)
    dirty_aabb = 1 << next(N)

    # Controle do mundo
    has_visualization = 1 << next(N)
    has_simple_visualization = 1 << next(N)
    has_solid_color = 1 << next(N)
    has_line_color = 1 << next(N)

    num_bits = next(N)
    full = (1 << num_bits) - 1
    del N

    @classmethod
    def _gencode(cls):
        '''Imprime código com a definição de cada flag. Poderia fazer
        dinamicamente, mas o Eclipse não entende e aponta erros que não
        existem.
        '''

        flags = {k: v for (k, v) in vars(cls).items() if not k.startswith('_')}
        del flags['num_bits']
        flags = ['%s = %s' % (k.upper(), v) for (k, v) in flags.items()]
        print('\n'.join(sorted(flags)))

    @classmethod
    def _make_negations(cls):
        '''Cria a negação de todas as flags'''

        cls.not_dirty_aabb = cls.full ^ cls.dirty_aabb


class BodyFlags(_BodyFlagsBase):
    f = _BodyFlagsBase

    # Negações das flags básicas ##############################################

    # Estado dinâmico do objeto
    not_active = f.full ^ f.active
    not_dynamic = f.full ^ f.is_dynamic
    not_kinematic = f.full ^ f.is_kinematic
    not_static = f.full ^ f.is_static
    not_sleep = f.full ^ f.is_sleeping
#     can_rotate = 1 << next(N)
#     can_sleep = 1 << next(N)
#
# Controle de forças
#     owns_gravity = 1 << next(N)
#     owns_damping = 1 << next(N)
#     owns_adamping = 1 << next(N)
#
# Contatos e vínculos
#     has_joints = 1 << next(N)
#     has_contacts = 1 << next(N)
#     has_external_force = 1 << next(N)
#     has_external_accel = 1 << next(N)
#     has_external_torque = 1 << next(N)
#     has_external_alpha = 1 << next(N)
#
# Estados temporários
    not_dirty_shape = f.full ^ f.dirty_shape
    not_dirty_aabb = f.full ^ f.dirty_aabb
#
# Controle do mundo
#     has_world = 1 << next(N)
#     has_visualization = 1 << next(N)

    # Flags compostas #########################################################
    dirty_any = f.dirty_aabb | f.dirty_shape
    not_dirty = f.full ^ dirty_any


del BodyFlags.f, count

###############################################################################
# Código copy & paste para do BodyFlags._gencode()
###############################################################################
CAN_ROTATE = 16
CAN_SLEEP = 32
HAS_CONTACTS = 1024
HAS_JOINTS = 512
HAS_VISUALIZATION = 8192
HAS_WORLD = 4096
IS_DIRTY = 2048
IS_DYNAMIC = 1
IS_KINEMATIC = 2
IS_SLEEP = 8
IS_STATIC = 4
OWNS_ADAMPING = 256
OWNS_DAMPING = 128
OWNS_GRAVITY = 64

###############################################################################
# Imprime o código quando chamado do __main__
###############################################################################
if __name__ == '__main__':
    _BodyFlagsBase._gencode()



'''
Nesta atividade vamos modificar a função de resolução de colisões da FGAme.

Modifique a função resolve_collision(col) abaixo para que ela implemente a
transferência de impulso usando as fórmulas aprendidas em sala de aula.
'''

from FGAme import *
from FGAme import patch


@patch.set_resolve_collision
def resolve_collision(col):
    '''Implementa uma rotina de resolução de colisão.

    O objeto de entrada `col` guarda várias informações importantes em seus
    atributos como o vetor normal em `col.normal`, o ponto de contato em
    `col.pos`, o coeficiente de restituição em `col.restitution` e a frição em
    `col.friction`.

    Documentação adicional sobre os objetos do tipo `Collision` pode ser
    encontrada no módulo :mod:`FGAme.physics.collision`. A rotina padrão para a
    resolução de colisões na FGAme pode ser acessada pelo método
    `col.resolve()`.
    '''

    # Este exemplo utiliza a estratégia ingênua de resolver as colisões apenas
    # invertendo as velocidades de cada objeto. Como podemos ver, isto não
    # funciona!
    #
    # Implemente aqui a rotina correta.
    A, B = col
    A.vel *= -1
    B.vel *= -1

# Aqui chamamos uma simulação já pronta para testar nosso método de resolução
# de colisões.
from FGAme.demos.simulations import gas_polys



# -*- coding: utf8 -*-
from __future__ import division
from math import sqrt
from FGAme.mathtools import Vec2, asvector


#
# Pontos de Contato/Manifolds
#
class ContactPoint(Vec2):

    '''Representa um ponto de contato com um certo nível de penetração'''

    __slots__ = ['depth']

    def __init__(self, x, y, depth):
        super(Vec2, self).__init__(x, y)
        self.depth = depth


class BaseContactManifold(object):

    '''Classe mãe para ContactManifold restitution SimpleContactManifold'''

    def __iter__(self):
        return iter(self.points)


class ContactManifold(BaseContactManifold):

    '''Representa uma lista de pontos de contato associados a uma única
    normal.'''

    __slots__ = ['normal', 'points']

    def __init__(self, normal, points):
        self.normal = normal.normalized()
        self.points = list(points)


class SimpleContactManifold(BaseContactManifold):

    '''Representa um contato simples com apenas um ponto'''

    __slots__ = ['normal', 'point']

    def __init__(self, normal, point):
        self.normal = normal
        self.point = point

    @property
    def points(self):
        return [self.point]

    def __iter__(self):
        yield self.point


#
# Contatos abstratos
#
class Pair(object):

    '''Representa alguma ligação, conexão ou agrupamento de dois objetos'''

    __slots__ = ('A', 'B')

    def __init__(self, A, B):
        self.A = A
        self.B = B

    def swap(self):
        '''Troca os dois objetos do par *INPLACE*'''

        self.B, self.A = self.A, self.B

    def swapped(self):
        '''Retorna um par com os elementos trocados'''

        new = self.copy()
        new.swap()
        return new

    def other(self, obj):
        '''Se for chamada com o objeto A, retorna o objeto B e vice-versa'''

        if obj is self.A:
            return self.B
        elif obj is self.B:
            return self.A
        else:
            raise ValueError('object does not participate in the connection')

    def isvalid(self):
        '''Retorna True caso o contato ainda seja válido ou False caso já tenha
        sido desfeito.'''

        raise NotImplementedError

    def update(self):
        '''Recalcula todas as propriedades que modem ser modificadas de um
        frame para o outro'''

        raise NotImplementedError

    #
    # Propriedades físicas
    #
    def center_of_mass(self):
        A, B = self
        return (A. mass * A.pos + B.mass * B.pos) / (A.mass + B.mass)

    def energyK(self):
        return self.A.energyK() + self.B.energyK()

    def momentumP(self):
        return self.A.momentumP() + self.B.momentumP()

    def momentumL(self):
        pos = (0, 0)
        return self.A.momentumL(pos) + self.B.momentumL(pos)

    #
    # Magic methods
    #
    def __eq__(self, other):
        return self is other

    def __getitem__(self, i):
        if i == 0:
            return self.A
        elif i == 1:
            return self.B
        else:
            raise IndexError(i)

    def __hash__(self):
        return id(self)

    def __iter__(self):
        yield self.A
        yield self.B

    def __contains__(self, element):
        return element is self.A or element is self.B


class BroadContact(Pair):

    '''Representa um contato na Broad phase'''

    def get_collision(self):
        '''Retorna o objecto de collisão associado ao contato. Caso não haja
        colisão, retorna None'''

        raise NotImplementedError


class CBBContact(BroadContact):

    '''Representa um par de objetos com caixas de contorno circulares que
    se sobrepõem'''

    pass


class AABBContact(BroadContact):

    '''Representa um par de objetos com caixas de contorno alinhadas ao eixo
    que se sobrepõem'''

    pass


###############################################################################
#                         Classe Colisão
# ----------------------------------------------------------------------------
#
# Representa uma colisão entre dois objetos.  Resolve a colisão sob demanda.
#
###############################################################################


class Collision(Pair):

    '''Representa a colisão entre dois objetos.

    Subclasses de Collision devem implementar o método .resolve() que resolve
    a colisão entre os dois objetos.

    Example
    -------

    Considere a colisão unidimensional entre dois círculos

    >>> from FGAme.physics import Circle, flags
    >>> A = Circle(1, pos=(0, 1), mass=1, vel=(1, 0))
    >>> B = Circle(1, pos=(2, 1), mass=1)

    A colisão unidimensional de dois objetos de mesma massa simplesmente troca
    as velocidades. Antes disso, no entanto, é necessário passar os parâmetros
    de colisão para que se possa calcular os resultados.

    >>> col = Collision(A, B,
    ...                 pos=(1, 1),    # ponto de colisão
    ...                 normal=(1, 0)) # normal da colisão

    Podemos investigar as propriedades físicas da colisão restitution verificar
    que as leis de conservação são preservadas

    >>> col.energyK(); col.momentumP(); col.momentumL()
    0.5
    Vec(1, 0)
    -1.0

    Podemos resolver a colisão utilizando ou o método step() ou o resolve(). O
    segundo é chamado apenas uma vez restitution já aciona os sinais de
    colisão. O método step() é utilizado em resoluções iterativas com vários
    objetos colidindo simultaneamente.

    >>> col.resolve()

    Agora investigamos os valores finais restitution vemos que as leis de
    conservação foram satisfeitas.

    >>> col.energyK(); col.momentumP(); col.momentumL()
    0.5
    Vec(1, 0)
    -1.0

    Investigando as propriedades, verificamos que de fato a velocidade dos dois
    objetos foi invertida

    >>> A.vel; B.vel
    Vec(0, 0)
    Vec(1, 0)


    Suponha agora que a colisão não se deu no ponto selecionado anteriormente.

    >>> A.vel, B.vel = (1, 0), (0, 0)
    >>> col = Collision(A, B,
    ...                 pos=(0, 0),    # ponto de colisão
    ...                 normal=(1, 0)) # normal da colisão

    Esta posição induz a uma velocidade de rotação e transferência de energia
    de movimento linear para angular.

    >>> col.resolve()
    >>> A.linearK() + B.linearK(), A.angularK() + B.angularK(), col.energyK()
    (0.2777777777777778, 0.2222222222222222, 0.5)

    As leis de conservação continuam sendo satisfeitas
    >>> col.energyK(); col.momentumP(); col.momentumL()
    0.5
    Vec(1, 0)
    -1.0
    '''

    def __init__(self, A, B, normal=None, pos=None, delta=0.0):
        super(Collision, self).__init__(A, B)
        self.normal = normal = asvector(normal)
        self.pos = pos = asvector(pos)
        self.delta = delta = float(delta)
        self.active = True

        # Obtêm parâmetros da colisão
        self.restitution = sqrt(A.restitution * B.restitution)
        self.friction = sqrt(A.friction * B.friction)

        # Posições e velocidades relativas
        rA = self.pos - A.pos
        rB = self.pos - B.pos

        # Massa efetiva
        invmass = A.invmass + B.invmass
        invmass += A.invinertia * (rA.cross(normal) ** 2)
        invmass += B.invinertia * (rB.cross(normal) ** 2)
        self.effmass = 1.0 / invmass

        # Asserções
        assert rA.dot(normal) >= 0, 'normal is pointing towards the first body'

    def resolve(self):
        A, B = self
        normal = self.normal
        pos = self.pos
        vrel = B.vpoint(pos) - A.vpoint(pos)
        vrel_normal = vrel.dot(normal)

        if vrel_normal < 0:
            # Impulso normal
            J = self.effmass * (1 + self.restitution) * vrel_normal
            Jvec = J * normal
            pos = self.pos

            # Impulso tangente

            A.apply_impulse(Jvec, pos=pos)
            B.apply_impulse(-Jvec, pos=pos)

    def apply_tangent(self, J):
        # Aplica incremento no impulso
        if J <= 0:
            return

        A, B = self.A, self.B
        vrel = self.vrel
        tangent = self.get_tangent()

        # Calcula o impulso tangente máximo
        vrel_tan = -vrel.dot(tangent)
        Jtan_max = abs(self.friction * J)

        # Limita a ação do impulso tangente
        if A._invmass and B._invmass:
            J = min(Jtan_max, vrel_tan / A._invmass, vrel_tan / B._invmass)
        elif A._invmass:
            J = min(Jtan_max, vrel_tan / A._invmass)
        elif B._invmass:
            J = min(Jtan_max, vrel_tan / B._invmass)
        else:
            J = 0.0

        Jvec = J * tangent
        if A._invmass:
            A.apply_impulse(-Jvec)
            if A._invinertia:
                A.apply_aimpulse(Jvec.cross(self.rA))
        if B._invmass:
            B.apply_impulse(Jvec)
            if B._invinertia:
                B.apply_aimpulse(-Jvec.cross(self.rB))

    def finalize(self, clear=True, trigger=True):
        if self.impulse > 0:
            self.apply_impulse(self.impulse)
        # self.apply_tangent(self.impulse)

        if trigger:
            # TODO: inspect collision triggers
            # self.A.trigger_collision(self)
            # self.B.trigger_collision(self)
            pass
        if clear:
            self.A.remove_contact(self)
            self.B.remove_contact(self)

    def get_tangent(self):
        # Vetor unitário tangente à colisão
        n = self.normal
        tangent = Vec2(-n.y, n.x)
        if tangent.dot(self.vrel) > 0:
            tangent = -tangent
        return tangent

    ###########################################################################
    #                       Métodos da API de Collision
    ###########################################################################
    def is_simple(self):
        '''Retorna True se o contato for o único contato de ambos os objetos
        envolvidos'''

        print('foo')
        return (len(self.A._contacts) <= 1) and (len(self.B._contacts) <= 1)

    def remove_overlap(self, beta=1.0):
        '''Remove a superposição entre os objetos movendo cada objeto por uma
        fração inversamente proporcional às respectivas massas'''

        normal = self.normal
        A, B = self
        a = A._invmass
        b = B._invmass
        beta /= a + b
        A.move((-beta * a) * normal)
        B.move((beta * b) * normal)

    def baumgarte(self, beta, dt, mindelta=0.3):
        delta = self.delta
        if delta > mindelta:
            A, B = self
            a = A._invmass
            b = B._invmass
            beta *= delta / (a + b) / dt
            a *= beta
            b *= beta
            A._e_vel -= a * self.normal
            B._e_vel += b * self.normal

    def baumgarte_adjust(self, beta, mindelta=0.3):
        delta = self.delta
        if delta > mindelta:
            A, B = self
            a = A._invmass
            b = B._invmass
            beta *= delta / (a + b)
            a *= beta
            b *= beta
            A.move(-a * self.normal)
            B.move(b * self.normal)


class ContactOrdered(Collision):

    '''Um objeto de contato em que o primeiro objeto é sempre mais pesado que
    o segundo'''

    def __init__(self, A, B, world=None, pos=None, normal=None, **kwds):
        if A._invmass > B._invmass:
            B, A = A, B
        super(ContactOrdered, self).__init__(A, B, world, pos, normal, **kwds)


class CollisionGroup(object):

    def __init__(self, collisions):
        pass


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# -*- coding: utf8 -*-

from FGAme import conf
from FGAme.objects import AABB, Rectangle
from FGAme.physics import Simulation
from FGAme.events import EventDispatcher, signal
from FGAme.draw import RenderTree, color_property
from FGAme.util import delegate_to
from FGAme.physics import Body
from FGAme.util import lazy


class World(EventDispatcher):

    '''Classe Mundo: coordena todos os objetos com uma física definida e
    resolve a interação entre eles.
    '''

    def __init__(self, background=None,
                 gravity=None, damping=0, adamping=0,
                 restitution=1, friction=0,
                 bounds=None, max_speed=None,
                 simulation=None, force_init=True):

        if force_init:
            conf.init()
            conf.show_screen()
        self.background = background
        self._render_tree = RenderTree()
        self._input = conf.get_input()

        if simulation:
            self._simulation = simulation
        else:
            self._simulation = Simulation(
                gravity=gravity,
                damping=damping,
                adamping=adamping,
                restitution=restitution,
                friction=friction,
                max_speed=max_speed,
                bounds=bounds)

        self.is_paused = False
        super(World, self).__init__()

        # Inicia e popula o mundo
        self.init()

    background = color_property('background', 'white')

    # Propriedades do objeto Simulation #######################################
    gravity = delegate_to('_simulation.gravity')
    damping = delegate_to('_simulation.damping')
    adamping = delegate_to('_simulation.adamping')
    time = delegate_to('_simulation.time', read_only=True)

    # Gerenciamento de objetos ################################################
    def add(self, obj, layer=0):
        '''Adiciona um novo objeto ao mundo.

        Exemplos
        --------

        >>> obj = AABB(-10, 10, -10, 10)
        >>> world = World()
        >>> world.add(obj, layer=1)
        >>> obj in world
        True

        Os objetos podem ser removidos com o método remove()

        >>> world.remove(obj)
        '''

        # Verifica se trata-se de uma lista de objetos
        if isinstance(obj, (tuple, list)):
            for obj in obj:
                self.add(obj, layer=layer)
            return

        # Objetos que implementam update_world()
        elif hasattr(obj, 'update_world'):
            obj.update_world(self, layer=layer)
            return

        # Adiciona na lista de renderização
        self._render_tree.add(obj, layer)
        if isinstance(obj, Body):
            self._simulation.add(obj)

    def remove(self, obj):
        '''Descarta um objeto do mundo'''

        self._render_tree.remove(obj)
        if isinstance(obj, Body):
            self._simulation.remove(obj)

    def init(self):
        '''Método executado após a inicialização do objeto World(). Normalmente
        é sobrescrito pelas sub-classes para popular o mundo com objetos e
        realizar outras operações de inicialização. A implementação padrão não
        faz nada.'''

    def __contains__(self, obj):
        return obj in self._render_tree or obj in self._simulation

    # Controle de eventos #####################################################

    # Delegações para o Input
    long_press = signal('long-press', 'key', delegate_to='_input')
    key_up = signal('key-up', 'key', delegate_to='_input')
    key_down = signal('key-down', 'key', delegate_to='_input')
    mouse_motion = signal('mouse-motion', delegate_to='_input')
    mouse_button_up = \
        signal('mouse-button-up', 'button', delegate_to='_input')
    mouse_button_down = \
        signal('mouse-button-down', 'button', delegate_to='_input')
    mouse_long_press = \
        signal('mouse-long-press', 'button', delegate_to='_input')

    # Delegação para o mainloop
    pre_draw = signal('pre-draw', num_args=1, delegate_to='_mainloop')
    post_draw = signal('post-draw', num_args=1, delegate_to='_mainloop')

    # Eventos privados
    frame_enter = signal('frame-enter')
    frame_skip = signal('frame-skip', num_args=1)
    collision = signal('collision', num_args=1)
    # TODO: collision_pair = signal('collision-pair', 'obj1', 'obj2',
    # num_args=1)

    # Simulação de Física #####################################################
    def pause(self):
        '''Pausa a simulação de física'''

        self.is_paused = True

    def unpause(self):
        '''Resume a simulação de física'''

        self.is_paused = False

    def toggle_pause(self):
        '''Alterna o estado de pausa da simulação'''

        self.is_paused = not self.is_paused

    def update(self, dt):
        '''Rotina principal da simulação de física.'''

        self.trigger('frame-enter')
        if not self.is_paused:
            self._simulation.update(dt)

    # Laço principal ##########################################################
    @lazy
    def _mainloop(self):
        return conf.get_mainloop()

    def run(self, timeout=None, real_time=True, **kwds):
        '''Roda a simulação de física durante o tempo 'timeout' especificado.

        O parâmetro `real_time` especifica se o tempo considerado consiste no
        tempo real ou no tempo de simulação.'''

        self._mainloop.run(self, timeout=timeout, **kwds)

    def stop(self):
        '''Finaliza o laço principal de simulação'''

        self._mainloop.stop()

    def set_next_state(self, value):
        '''Passa a simulação para o próximo estado'''

        pass

    def get_render_tree(self):
        return self._render_tree

    ###########################################################################
    #                     Criação de objetos especiais
    ###########################################################################
    def add_bounds(self, *args, **kwds):
        '''Cria um conjunto de AABB's que representa uma região fechada.

        Parameters
        ----------

        '''

        # Processa argumentos
        hard = kwds.pop('hard', True)
        delta = kwds.pop('delta', 10000)
        use_poly = kwds.pop('use_poly', False)

        if len(args) == 4:
            xmin, xmax, ymin, ymax = args
        elif len(args) == 1:
            xmin, xmax, ymin, ymax = args[0]
        elif not args:
            if 'width' not in kwds:
                raise TypeError('not enougth parameters to set boundaries')

            W, H = conf.get_resolution()
            value = kwds.pop('width')
            try:
                N = len(value)
                if N == 2:
                    dx, dy = value
                    xmin, xmax = dx, W - dx
                    ymin, ymax = dy, H - dy
                elif N == 4:
                    dx, dy, dx1, dy1 = value
                    xmin, xmax = dx, W - dx1
                    ymin, ymax = dy, H - dy1
                else:
                    raise ValueError('width can have 1, 2 or 4 values')
            except TypeError:
                dx = dy = value
                xmin, xmax = dx, W - dx
                ymin, ymax = dy, H - dy
        else:
            raise TypeError('invalid number of positional arguments')

        assert xmin < xmax and ymin < ymax, 'invalid bounds'
        maker = Rectangle if use_poly else AABB

        up = maker(
            bbox=(xmin - delta, xmax + delta, ymax, ymax + delta), **kwds)
        down = maker(
            bbox=(xmin - delta, xmax + delta, ymin - delta, ymin), **kwds)
        left = maker(
            bbox=(xmin - delta, xmin, ymin, ymax), **kwds)
        right = maker(
            bbox=(xmax, xmax + delta, ymin, ymax), **kwds)

        for box in [up, down, left, right]:
            box.make_static()
            assert box._invmass == 0.0
        self.add([up, down, left, right])
        self._bounds = (left, right, up, down)
        self._hard_bounds = hard

    ###########################################################################
    #                         Funções úteis
    ###########################################################################
    def register_energy_tracker(self, auto_connect=True, ratio=True,
                                raise_on_change=None):
        '''Retorna uma função que rastreia a energia total do mundo a cada
        frame e imprime sempre que houver mudança na energia total.

        O comportamento padrão é imprimir a razão entre a energia inicial e a
        atual. Caso ``ratio=False`` imprime o valor da energia em notação
        científica.

        A função resultante é conectada automaticamente ao evento
        "frame-enter", a não ser que ``auto_conect=False``.
        '''

        last_data = [None]
        if raise_on_change is None:
            raise_on_change = conf.DEBUG

        if ratio:
            def energy_tracker():
                last = last_data[0]
                total = self._simulation.energy_ratio()
                if (last is None) or (abs(total - last) > 1e-6):
                    msg = 'Energia total / energia inicial: %s' % total

                    if (last is not None) and raise_on_change:
                        raise ValueError(msg)
                    else:
                        if not conf.DEBUG:
                            print(msg)
                    last_data[0] = total
        else:
            def energy_tracker():
                raise NotImplementedError

        if auto_connect:
            self.listen('frame-enter', energy_tracker)

        return energy_tracker


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# -*- coding: utf8 -*-
'''
As funções no módulo patch permitem modificar o comportamento da FGAme de
maneira arbitrária. A maior parte das funções tem o caráter educativo e
permitem que estudantes na área de física para jogos reimplementem
pedaços específucis de um motor de jogo aproveitando o resto da infraestrutura
existente.

As funções neste módulo também servem para testar implementações alternativas
ou experimentais.
'''
from types import MethodType


def set_collision_class(cls):
    '''Define qual é a classe encarregada de resolver colisões no lugar de
    FGAme.physics.Collision.'''

    from FGAme.physics import collision_pairs
    collision_pairs.Collision = cls


def restore_collision_class():
    '''Restaura a classe de colisão para o valor padrão'''

    from FGAme.physics import Collision
    set_collision_class(Collision)


def set_resolve_collision(func):
    '''Decorador que determina qual é a função encarregada de resolver colisões
    entre duas partículas. A função recebe um objeto de colisão como argumento.

        >>> @set_resolve_collision
        ... def resolve(col):
        ...     A, B = col
        ...     print('collision with objects %s and %s' % (A, B))

    Note que é possível invocar a implementação original do método de colisão
    utilizando o método col.resolve().
    '''

    from FGAme.physics import Collision

    class PatchedCollision(Collision):

        def __init__(self, *args, **kwds):
            try:
                super(PatchedCollision, self).__init__(*args, **kwds)
            except AssertionError:
                pass

        def resolve(self):
            try:
                self.resolve = MethodType(Collision.resolve, self)
                func(self)
            finally:
                del self.resolve

    set_collision_class(PatchedCollision)


def restore_defaults():
    '''Restaura a FGAme para o comportamento padrão'''

    restore_collision_class()



# -*- coding: utf8 -*-

from collections import defaultdict
from FGAme.mathtools import Vec2, null2D
from FGAme.events import EventDispatcher, signal
from FGAme.physics.flags import BodyFlags
from FGAme.physics.broadphase import BroadPhase, BroadPhaseCBB, NarrowPhase

###############################################################################
#                                Simulação
# ----------------------------------------------------------------------------
# Coordena todos os objetos com uma física definida e resolve a interação
# entre eles
###############################################################################
SLEEP_LINEAR_VELOCITY = 3
SLEEP_ANGULAR_VELOCITY = 0.05


class Simulation(EventDispatcher):

    '''Implementa a simulação de física.

    Os métodos principais são: add(obj) e remove(obj) para adicionar e remover
    objetos e update(dt) para atualizar o estado da simulação. Verifique a
    documentação do método update() para uma descrição detalhada sobre como
    a física é resolvida em cada etapa de simulação.
    '''

    def __init__(self, gravity=None, damping=0, adamping=0,
                 restitution=1, friction=0, max_speed=None,
                 bounds=None, broad_phase=None, niter=5, beta=0.0):

        super(Simulation, self).__init__()

        # Listas de objetos e vínculos
        self._objects = []
        self._constraints = []
        self._contacts = []
        self._inactive = []

        # Parâmetros do solver
        self.niter = niter
        self.beta = beta

        # Define algortimos
        self.broad_phase = normalize_broad_phase(broad_phase, self)
        self.narrow_phase = NarrowPhase(world=self)

        # Inicia parâmetros físicos
        self._kinetic0 = None
        self._potential0 = None
        self._interaction0 = None
        self._gravity = null2D
        self._damping = self._adamping = self._friction = 0
        self._restitution = 1
        self.gravity = gravity or (0, 0)
        self.damping = damping
        self.adamping = adamping
        self.restitution = restitution
        self.friction = friction
        self.max_speed = max_speed

        # Limita mundo
        self.bounds = bounds
        self._out_of_bounds = set()

        # Inicializa constantes de simulação
        self.num_steps = 0
        self.time = 0

    ###########################################################################
    #                           Serviços Python
    ###########################################################################
    def __iter__(self):
        return iter(self._objects)

    def __contains__(self, obj):
        return obj in self._objects

    ###########################################################################
    #                               Sinais
    ###########################################################################
    frame_enter = signal('frame-enter')
    collision = signal('collision', num_args=1)
    object_add = signal('object-add', num_args=1)
    object_remove = signal('object-remove', num_args=1)
    gravity_change = signal('gravity-change', num_args=2)
    damping_change = signal('damping-change', num_args=2)
    adamping_change = signal('adamping-change', num_args=2)
    restitution_change = signal('restitution-change', num_args=2)
    friction_change = signal('friction-change', num_args=2)

    ###########################################################################
    #                       Propriedades físicas
    ###########################################################################
    @property
    def gravity(self):
        return self._gravity

    @gravity.setter
    def gravity(self, value):
        owns_prop = BodyFlags.owns_gravity
        old = self._gravity
        try:
            gravity = self._gravity = Vec2(*value)
        except TypeError:
            gravity = self._gravity = Vec2(0, -value)

        for obj in self._objects:
            if not obj.flags & owns_prop:
                obj._gravity = gravity
        self.trigger('gravity-change', old, self._gravity)

    @property
    def damping(self):
        return self._damping

    @damping.setter
    def damping(self, value):
        owns_prop = BodyFlags.owns_damping
        old = self._damping
        value = self._damping = float(value)

        for obj in self._objects:
            if not obj.flags & owns_prop:
                obj._damping = value
        self.trigger('damping-change', old, self._damping)

    @property
    def adamping(self):
        return self._adamping

    @adamping.setter
    def adamping(self, value):
        owns_prop = BodyFlags.owns_adamping
        old = self._adamping
        value = self._adamping = float(value)

        for obj in self._objects:
            if not obj.flags & owns_prop:
                obj._adamping = value
        self.trigger('adamping-change', old, self._damping)

    @property
    def restitution(self):
        return self._restitution

    @restitution.setter
    def restitution(self, value):
        owns_prop = BodyFlags.owns_restitution
        old = self._restitution
        value = self._restitution = float(value)

        for obj in self._objects:
            if not obj.flags & owns_prop:
                obj._restitution = value
        self.trigger('restitution-change', old, self._restitution)

    @property
    def friction(self):
        return self._friction

    @friction.setter
    def friction(self, value):
        owns_prop = BodyFlags.owns_friction
        old = self._friction
        value = self._friction = float(value)

        for obj in self._objects:
            if not obj.flags & owns_prop:
                obj._friction = value
        self.trigger('friction-change', old, self._friction)

    ###########################################################################
    #                   Gerenciamento de objetos e colisões
    ###########################################################################
    def add(self, obj):
        '''Adiciona um novo objeto ao mundo.

        Exemplos
        --------

        >>> from FGAme import *
        >>> obj = AABB(bbox=(-10, 10, -10, 10))
        >>> world = World()
        >>> world.add(obj, layer=1)
        '''

        flags = BodyFlags

        if obj not in self._objects:
            self._objects.append(obj)
            obj._world = self

            oflags = obj.flags
            if not oflags & flags.owns_gravity:
                obj._gravity = self.gravity
            if not oflags & flags.owns_damping:
                obj._damping = self.damping
            if not oflags & flags.owns_adamping:
                obj._adamping = self.adamping
            if not oflags & flags.owns_restitution:
                obj._restitution = self.restitution
            if not oflags & flags.owns_friction:
                obj._friction = self.friction
            self.trigger('object-add', obj)

    def remove(self, obj):
        '''Remove um objeto da simulação. Produz um ValueError() caso o objeto
        não esteja presente na simulação.'''

        try:
            idx = self._objects.index(obj)
        except IndexError:
            raise ValueError('object not present')
        else:
            del self._objects[idx]
            self.trigger('object-remove', obj)
            obj.destroy()

    def discard(self, obj):
        '''Descarta um objeto da simulação.'''

        try:
            self.remove(obj)
        except ValueError:
            pass

    ###########################################################################
    #                     Simulação de Física
    ###########################################################################
    def update(self, dt):
        '''Rotina principal da simulação de física.'''

        self.trigger_frame_enter()
        self._dt = dt = float(dt)

        # Inicializa energia
        if self._kinetic0 is None:
            self._init_energy0()

        # Loop genérico
        self.accumulate_accelerations(dt)
        self.resolve_velocities(dt)
        self.resolve_constraints(dt)  # Colisão é um tipo de vínculo!
        self.resolve_positions(dt)

        # Incrementa tempo e contador
        self.time += dt
        self.num_steps += 1

        # Serviços esporáticos que não são realizados em todos os frames
        if self.num_steps % 2 == 0:
            self.find_out_of_bounds()
        elif self.num_steps % 2 == 1:
            self.enforce_max_speed()

    def accumulate_accelerations(self, dt):
        '''Atualiza o vetor interno que mede as acelerações lineares e
        angulares de cada objeto.

        Para tanto, usa informação tanto de forças globais quanto dos atributos
        *force* e *torque* de cada objeto.'''

        IS_SLEEP = BodyFlags.is_sleeping
        t = self.time

        # Acumula as forças e acelerações
        for obj in self._objects:
            if obj.flags & IS_SLEEP:
                continue

            if obj._invmass:
                obj.init_accel()
                if obj.force is not None:
                    obj._accel += obj.force(t) * obj._invmass

            # elif obj.flags & ACCEL_STATIC:
            #    obj.init_accel()
            #    obj.apply_accel(obj._accel, dt)

            if obj._invinertia:
                obj.init_alpha()
                if obj.torque is not None:
                    obj._alpha += obj.torque(t) * obj._invinertia

            # elif obj.flags & ALPHA_STATIC:
            #    obj.init_alpha()
            #    obj.apply_alpha(self._alpha, dt)

    def resolve_velocities(self, dt):
        '''Calcula as novas velocidades em função das acelerações acumuladas no
        passo accumulate_accelerations'''

        IS_SLEEP = BodyFlags.is_sleeping
        for obj in self._objects:
            if obj.flags & IS_SLEEP:
                continue

            if obj._invmass:
                obj.boost(obj._accel * dt)
            if obj._invinertia:
                obj.aboost(obj._alpha * dt)

            obj._e_vel = null2D
            obj._e_omega = 0.0

    def resolve_positions(self, dt):
        '''Resolve as posições a partir das velocidades'''

        IS_SLEEP = BodyFlags.is_sleeping
        for obj in self._objects:
            if obj.flags & IS_SLEEP:
                continue
            obj.move((obj.vel + obj._e_vel) * dt)
            obj.rotate((obj.omega + obj._e_omega) * dt)

    def resolve_constraints(self, dt):
        '''Resolve todos os vínculos utilizando o algoritmo de impulsos
        sequenciais'''

        broad_cols = self.broad_phase(self._objects)
        narrow_cols = self.narrow_phase(broad_cols)

        for col in narrow_cols:
            col.resolve()

        # Estabiliza contatos usando a estabilização de baumgarte
        #beta = self.beta
        # for col in narrow_cols:
        #    col.baumgarte_adjust(beta)

    def get_islands(self, contacts):
        '''Retorna a lista de grupos de colisão fechados no gráfico de
        colisões'''

        contacts = set(contacts)
        groups = defaultdict(list)
        while contacts:
            A, B = C = contacts.pop()
            gA = groups[A]
            gB = groups[B]
            if gA is not gB:
                gA.extend(gB)
                groups[B] = gA
            gA.append(C)
        return list(groups.values())

    def can_collide(self, A, B):
        '''Retorna True se A e B podem colidir'''

        flags = BodyFlags

        # Testa se os dois objetos são estáticos
        if ((not A._invmass or A.flags & flags.is_sleeping) and
                (not B._invmass or B.flags & flags.is_sleeping)):
            return False

        # Testa se estão na mesma camada de colisão
        elif ((A._col_layer_mask != B._col_layer_mask)
              and not (A._col_layer_mask & B._col_layer_mask)):
            return False

        # Testa se estão no mesmo grupo de colisão mesma camada de colisão
        elif A._col_group_mask & B._col_group_mask:
            return False

        return True

    #
    # Cálculo de parâmetros físicos
    #
    def energyK(self):
        '''Soma da energia cinética de todos os objetos do mundo'''

        return sum(obj.energyK() for obj in self._objects
                   if (obj._invmass or obj._invinertia))

    def energyU(self):
        '''Soma da energia potencial de todos os objetos do mundo devido à
        gravidade'''

        return sum(obj.energyU() for obj in self._objects if obj._invmass)

    def energy_interaction(self):
        '''Soma da energia de interação entre todos os pares de partículas
        (Não implementado)'''

        return 0.0

    def energyT(self):
        '''Energia total do sistema de partículas (possivelmente excluindo
        algumas interações entre partículas)'''

        return self.energyU() + self.energyK() + self.energy_interaction()

    def energy_ratio(self):
        '''Retorna a razão entre a energia total e a energia inicial calculada
        no início da simulação'''

        if self._kinetic0 is None:
            self._init_energy0()
            return 1.0
        sum_energies = self._kinetic0 + self._potential0 + self._interaction0
        return self.energyT() / sum_energies

    def _init_energy0(self):
        '''Chamada para inicializar _kinetic0 e amigos'''

        self._kinetic0 = self.energyK()
        self._potential0 = self.energyU()
        self._interaction0 = self.energy_interaction()

    #
    # Serviços esporáticos
    #
    def enforce_max_speed(self):
        '''Força que todos objetos tenham uma velocidade máxima'''

        if self.max_speed is not None:
            vel = self.max_speed
            vel_sqr = self.max_speed ** 2

            for obj in self._objects:
                if obj._vel.norm_sqr() > vel_sqr:
                    obj._vel *= vel / obj._vel.norm()

    def find_out_of_bounds(self):
        '''Emite o sinal de "out-of-bounds" para todos os objetos da
        simulação que deixarem os limites estabelecidos'''

        if self.bounds is not None:
            xmin, xmax, ymin, ymax = self.bounds
            out = self._out_of_bounds

            for obj in self._objects:
                x, y = obj._pos
                is_out = True

                if x > xmax and obj.xmin > xmax:
                    direction = 0
                elif y > ymax and obj.ymin > ymax:
                    direction = 1
                elif x < xmin and obj.xmax < xmin:
                    direction = 2
                elif y < ymin and obj.ymax < ymin:
                    direction = 3
                else:
                    is_out = False

                if is_out and obj not in out:
                    out.add(obj)
                    obj.trigger_out_of_bounds(direction)
                else:
                    out.discard(obj)

    def burn(self, frames, dt=0.0):
        '''Executa a simulação por um número específico de frames sem deixar
        o tempo rodar'''

        time = self.time
        for _ in range(frames):
            self.update(dt)
            self.time = time

    def remove_superpositions(self, num_iter=1):
        '''Remove todas as superposições entre objetos dinâmicos'''

        self._dt = 0.0
        for _ in range(num_iter):
            self.broad_phase()
            self.fine_phase()

            for col in self._fine_collisions:
                col.adjust_overlap()


###############################################################################
#                              Funções auxiliares
###############################################################################
def normalize_broad_phase(broad_phase, world):
    '''Escolhe o parâmetro correto na inicialização do broad-phase'''

    if broad_phase is None:
        broad_phase = BroadPhaseCBB(world=world)
    elif isinstance(broad_phase, BroadPhase):
        if broad_phase.world not in [None, world]:
            raise ValueError('BroadPhase object has a world attatched')
        else:
            broad_phase.world = world
    elif isinstance(broad_phase, type) and issubclass(broad_phase, BroadPhase):
        broad_phase = broad_phase(world=world)
    else:
        raise TypeError('invalid broad phase object')
    return broad_phase

if __name__ == '__main__':
    import doctest
    doctest.testmod()



#-*-coding: utf8 -*-
from FGAme import conf, pi
from FGAme.draw import *
from time import sleep

# Cria a árvore de renderização com vários círculos
p0 = (400, 300)
tree = RenderTree()
tree.add(Circle(150, p0, color='red', line_width=10.0, line_color='blue'))
tree.add(Rectangle(rect=(100, 100, 200, 200), theta=pi / 3, color='blue'))
tree.add(Poly([(400, 200), (800, 200), (600, 500)], color='black'))
tree.add(Segment((0, 0), (800, 600), color='red', width=20))
tree.add(
    Path([(0, 0), (200, 300), (300, 50), (800, 600)], color='red', width=20))

# Cria um objeto Canvas com a geometria da tela
canvas = conf.init_screen(800, 600)
canvas.show()
white = Color('white')

# Loop principal
for i in range(50):
    canvas.clear_background(white)
    canvas.draw(tree)
    canvas.flip()
    sleep(0.05)



from pygame import gfxdraw  # @UnresolvedImport
from FGAme.draw import Color
from FGAme.backends.pygame_be import PyGameCanvas, PyGameInput, PyGameMainLoop

black = Color('black')


class PyGameGFXCanvas(PyGameCanvas):

    '''Implementa a interface Canvas, utilizando a biblioteca Pygame e as
    funções pygame.gfxdraw, ao invés das usuais pygame.draw'''

    _gfx_circle_border = gfxdraw.aacircle
    _gfx_circle = gfxdraw.filled_circle
    _gfx_poly_border = gfxdraw.aapolygon
    _gfx_poly = gfxdraw.filled_polygon
    _gfx_line = gfxdraw.line

    def draw_raw_circle_solid(self, circle, color=black):
        x, y = map(int, circle.pos)
        y = self.height - y
        r = int(circle.radius)
        self._gfx_circle(self._screen, x, y, r, color)
        self._gfx_circle_border(self._screen, x, y, r, color)

    def draw_raw_circle_border(self, circle, width=1.0, color=black):
        if width == 1.0:
            x, y = map(int, circle.pos)
            y = self.height - y
            r = int(circle.radius)
            self._gfx_circle_border(self._screen, x, y, r, color)
        else:
            super(PyGameGFXCanvas, self) \
                .draw_raw_circle_border(circle, width, color)

    def draw_raw_poly_solid(self, poly, color=black):
        Y = self.height
        vertices = [(int(x), int(Y - y)) for (x, y) in poly]
        self._gfx_poly_border(self._screen, vertices, color)
        self._gfx_poly(self._screen, vertices, color)

    def draw_raw_poly_border(self, poly, width=1.0, color=black):
        if width == 1.0:
            Y = self.height
            vertices = [(int(x), int(Y - y)) for (x, y) in poly]
            self._gfx_poly_border(self._screen, vertices, color)
        else:
            super(PyGameGFXCanvas, self) \
                .draw_raw_poly_border(poly, width, color)

    def draw_raw_segment(self, segment, width=1.0, color=black):
        Y = self.height
        pt1, pt2 = [(int(x), int(Y - y)) for (x, y) in segment]
        x1, y1 = pt1
        x2, y2 = pt2
        self._gfx_line(self._screen, x1, y1, x2, y2, color)



'''
Parsers para micro-linguagens baseadas em indentação.
'''
import re
from collections import namedtuple

#
# Regex constants
#
WHITESPACE = re.compile('\s+')
B_WHITESPACE = re.compile('^\s+')


Node = namedtuple('Node', ['head', 'body', 'nodes', 'line_no'])


class rule(object):

    def __init__(self, func, is_optional=False):
        self.is_optional = is_optional
        self._func
        self._node = None
        self._result = None

    def is_valid(self, node):
        try:
            self._result = self._func(node)
            self._node = node
            return True
        except ValueError:
            return False

    def parse(self, node):
        if node is self._node:
            return self._result
        elif not self.is_valid(node):
            raise ValueError('cannot parse an invalid node')
        else:
            return self._result

    @classmethod
    def from_keys(self, **kwds):
        pass


class IndentParser(object):

    def __init__(self):
        self.rules = []

    def parse(self, source):
        return blockparse(source)

    def __call__(self, *args, **kwds):
        return self.parse(*args, **kwds)


def blockparse(source):
    '''Retorna uma lista de nós do tipo Node, onde cada nó possui a
    estrutura::

        <one line head>
            <multi line body>
    '''
    blocks = []
    indent = 0
    source = deindent(normalize_blank_lines(source))

    # Cria blocos
    for line_no, line in enumerate(source.splitlines()):
        if line:
            indent = get_indent(line)
            if not indent:
                block = Node(line, [], None, line_no)
                blocks.append(block)
            else:
                body = blocks[-1].body
                if body and get_indent(body[-1]) != indent:
                    raise SyntaxError('inconsistent indentation')
                body.append(line)
        elif blocks:
            blocks[-1]

    # Funde lista de linhas para cada bloco
    for idx, block in enumerate(blocks):
        head, data, _, line_no = block
        data = deindent('\n'.join(data))
        try:
            nodes = blockparse(data)
        except SyntaxError:
            nodes = None
        blocks[idx] = Node(head, data, nodes, line_no)

    return blocks


def normalize_blank_lines(source):
    '''Transformas linhas que contêm apenas espaços em linhas vazias'''

    lines = source.splitlines()
    lines = [('' if line.isspace() else line) for line in lines]
    return '\n'.join(lines)


def normalize_tabs(source, size=4):
    '''Troca tabs por espaços'''

    return source.replace('\t', ' ' * size)


def deindent(source):
    '''Remove uma indentação global'''

    # Normaliza
    source = normalize_blank_lines(normalize_tabs(source))

    # Encontra indentação
    indent = float('inf')
    lines = source.splitlines()
    for line in lines:
        if line:
            match = B_WHITESPACE.match(line)
            if match:
                indent = min(indent, match.end())

    # Reindenta todas as linhas
    if indent == float('inf'):
        indent = 0
    for i, line in enumerate(lines):
        if line:
            lines[i] = line[indent:]

    return '\n'.join(lines)


def get_indent(st):
    '''Retorna a indentação da string de entrada'''

    for idx, c in enumerate(st):
        if c != ' ':
            return idx
    return 0


source = '''
    Group:
        color: red

        Circle:
            radius: 10
            pos: middle

        Circle:
            radius: 10
            pos: middle + (100, 100)


    foo:
        arg1, arg2, arg3
        noarg
        arg4

        foo: bar

        @long-press(up):
            self.move(0, 10)

        @long-press(down):
            self.move(0, -10)

    bar:
        sdfsdfsd sdfsdf
'''

p = IndentParser()
print(p.parse(source))



# -*- coding: utf8 -*-
'''
A FGAme permite modificar os parâmetros de inicialização utilizando as funções
do módulo :mod:`FGAme.conf`. Nesta estão funções que controlam a inicialização
do motor de jogos e permitem recuperar o valor de alguns parâmetros globais.

Note que quase todas as funções que modificam o estado global contidas neste
módulo só podem ser executadas antes de iniciar o loop principal do jogo e de
definir eventos e callbacks para objetos. Modificar uma propriedade após a
inicialização quase sempre resultará em um erro.

A inicialização pode ser feita explicitamente invocando a função
:func:`FGAme.init()` ou implicitamente, quando executamos o loop principal. De
um modo geral, o programa deve ser organizado assim::

    from FGAme import *

    # Configurações
    conf.set_resolution(1024, 764)
    conf.set_backend('sdl2')
    ...
    init()

    # Cria objetos
    world = World()
    ...

    # Inicia mundo
    world.run()
'''

from FGAme.core import log as _log
from FGAme import backends as _backend_module

# --------------------------------------------------------------------------- #
#                       Variáveis globais para o módulo
# --------------------------------------------------------------------------- #
DEBUG = False
_has_init = False
_app_name = None
_conf_path = None
_input_class = None
_input_object = None
_physics_fps = 60
_physics_dt = 1 / 60.
_mainloop_class = None
_mainloop_object = None
_screen_class = None
_screen_object = None
_window_origin = (0, 0)
_window_shape = None
_backend = None
_backends = ['sdl2', 'pygamegfx', 'pygame', 'sdl2cffi']
_backend_classes = None


#
# Configuram parâmetros da simulação
#
def set_resolution(*args):
    '''Configura a tela com a resolução dada.

    conf.set_resolution(800, 600)     --> define o formato em pixels
    conf.set_resolution('fullscreen') --> modo tela cheia'''

    global _window_shape

    if len(args) == 2:
        width, height = args
    elif args == ('fullscreen',):
        raise NotImplementedError
    elif len(args) == 1:
        width, height = args[0]
    else:
        raise TypeError('invalid arguments: %s' % args)

    if _window_shape is None or _window_shape == (width, height):
        _window_shape = (width, height)
    else:
        raise RuntimeError(
            'cannot change resolution after screen initialization')


def set_framerate(value):
    '''Configura o número de frames por segundo'''

    global _physics_dt, _physics_fps
    value = float(value)
    _physics_fps = value
    _physics_dt = 1.0 / value


def set_frame_duration(value):
    '''Configura a duração de um frame de simulação'''

    set_framerate(1.0 / value)


def set_backend(backend=None):
    '''Define o backend a ser utilizado pela FGAme.

    Se for chamada sem nenhum argumento, tenta carregar os backends na
    ordem padrão. Se o argumento for uma lista, tenta carregar os backends na
    ordem especificada pela lista. Retorna uma string descrevendo o backend
    carregado.'''

    global _backend, _backend_classes

    # Função chamada sem argumentos
    if backend is None:
        if _backend is None:
            backend = _backends
        else:
            return  # backend já configurado!

    # Previne modificar o backend
    if _backend is not None:
        if _backend != backend:
            raise RuntimeError(
                'already initialized to %s, cannot change the backend' %
                _backend)
        else:
            return

    # Carrega backend pelo nome
    if isinstance(backend, str):
        if not _backend_module.supports_backend(backend):
            raise ValueError(
                '%s backend is not supported in your system' %
                backend)

        _backend = backend
        _backend_classes = _backend_module.get_backend_classes(backend)
        _log.info('conf: Backend set to %s' % backend)

    # Carrega backend a partir de uma lista
    else:
        for be in backend:
            if _backend_module.supports_backend(be):
                set_backend(be)
                break
        else:
            msg = 'none of the requested backends are available.'
            if 'pygame' in backend:
                msg += (
                    '\nSupported backends are:'
                    '\n    * pygame'
                    '\n    * sdl2'
                    # '\n    * kivy'
                )
            raise RuntimeError(msg)
    return _backend


def get_backend(force=False):
    '''Retorna uma string com o backend incializado.

    Caso nenhum backend tenha sido inicializado, retorna None. Se o argumento
    `force` for verdadeiro, força a inicialização de algum backend.'''

    if force and _backend is None:
        return set_backend()
    return _backend

#
# Funções de inicialização
#


def init():
    '''Inicializa todas as classes relevantes do FGAme.

    Força a inicialização na primeira chamada. As chamadas subsequentes não
    produzem efeito.'''

    global _has_init

    # Previne a inicialização repetida
    if _has_init:
        return

    # Garante que exite um backend definido
    set_backend()

    # Inicializa o vídeo, input, mainloop, etc
    if _screen_object is None:
        init_screen()

    if _input_object is None:
        init_input()

    if _mainloop_object is None:
        init_mainloop()

    _has_init = True


def init_mainloop(screen=None, input=None, fps=None):  # @ReservedAssignment
    '''Inicializa o objeto de loop principal.

    Note que esta função não executa o loop principal, mas apenas instancia o
    objeto que irá coordenar o loop principal quando uma função como
    :meth:`World.run` for chamada.
    '''

    global _mainloop_class, _mainloop_object

    if _mainloop_class is None:
        set_backend()
        _mainloop_class = _backend_classes['mainloop']

    _set_screen(screen)
    _set_input(screen)
    _set_fps(fps)
    screen = screen or get_screen()
    input_ = input or get_input()
    fps = fps or get_framerate()
    _mainloop_object = _mainloop_class(screen, input_, fps)
    return _mainloop_object


def init_input():
    '''Inicializa o sistema de inputs de teclado e mouse.'''

    global _input_object, _input_class

    if _input_class is None:
        set_backend()
        _input_class = _backend_classes['input']

    _input_object = _input_class()
    return _input_object


def init_screen(*args, **kwds):
    '''Inicializa a tela na resolução padrão de 800x600 ou na resolução
    especificada pelo usuário usando a função `set_resolution()`'''

    global _screen_class, _screen_object

    if _screen_object is not None:
        raise RuntimeError('trying to re-init screen object')

    if _screen_class is None:
        set_backend()
        _screen_class = _backend_classes['screen']

    if not args:
        shape = get_resolution()
        screen = _screen_class(shape=shape, **kwds)
    elif len(args) == 2:
        screen = _screen_class(shape=args, **kwds)
    elif args == ('fullscreen',):
        screen = _screen_class(shape=args, **kwds)

    # Save environment variables
    if _window_shape is None:
        set_resolution(screen.shape)
    _screen_object = screen
    screen.init()
    return screen


#
# Retornam objetos e estados de simulação
#
def get_mainloop():
    '''Retorna o objeto screen inicializado'''

    return _mainloop_object or init_mainloop()


def get_input():
    '''Retorna o objeto screen inicializado'''

    return _input_object or init_input()


def get_screen():
    '''Retorna o objeto screen inicializado'''

    return _screen_object or init_screen()


def get_resolution():
    '''Retorna uma tupla com a (largura, altura) da tela em pixels'''

    return _window_shape or (800, 600)


def get_framerate():
    '''Retorna a taxa de atualização global em frames por segundo'''

    return _physics_fps


def get_frame_duration():
    '''Retorna a duração de cada frame (inverso do framerate) em segundos'''

    return _physics_dt


#
# Outras funções
#
def show_screen():
    '''Mostra a tela principal.

    Inicia com os parâmetros definidos, caso a tela ainda não tenha sido
    criada'''

    (_screen_object or init_screen()).show()


#
# Funções privadas
#
def _set_var_worker(var, force=False):
    def func(value):
        G = globals()
        obj = G[var]
        if value is not None:
            if obj is None or obj is value or force:
                obj = value
            else:
                name = var.strip('_')
                if name.endswith('_object'):
                    name = name[:-7]
                name = name.replace('_', ' ')
                raise RuntimeError('reseting %s object' % name)
    return func

_set_screen = _set_var_worker('_screen_object')
_set_input = _set_var_worker('_input_object')
_set_mainloop = _set_var_worker('_mainloop_object')
_set_fps = _set_var_worker('_physics_fps', force=True)



#-*- coding: utf8 -*-
'''
Importa as funções matemáticas do módulo mathutils.
'''

from __future__ import division
from math import *
from smallvectors import *
import smallvectors as linalg
from smallshapes import *
import smallshapes as shapes
from generic import set_promotion_rule, set_conversion

#
# Apelidos para tipos vetoriais
#
Point2 = Point[2, float]
Point3 = Point[3, float]
Point4 = Point[4, float]


class FGAmeVec(object):
    '''A FGAme vector object'''

    def __repr__(self):
        data = ['%.1f' % x for x in self]
        if all(x.endswith('.0') for x in data):
            data = [x[:-2] for x in data]
        return 'Vec(%s)' % (', '.join(data))

    def __radd__(self, other):
        return type(self)(*(y + x for (x, y) in zip(self, other)))

    def __rsub__(self, other):
        return type(self)(*(y - x for (x, y) in zip(self, other)))

    def __rmul__(self, other):
        return type(self)(*(x * other for x in self))

    def __mul__(self, other):
        return type(self)(*(x * other for x in self))

    def __rtruediv__(self, other):
        return type(self)(*(x / other for x in self))


class Vec2(FGAmeVec, Vec[2, float]):
    '''Vetor bidimensional'''


@set_conversion(list, Vec2)
@set_conversion(tuple, Vec2)
def _tuple2vec2(u):
    x, y = u
    return Vec2(x, y)


set_promotion_rule(Vec2, tuple, Vec2)
set_promotion_rule(Vec2, list, Vec2)


@add.overload([Vec2, tuple])
@add.overload([Vec2, list])
@add.overload([tuple, Vec2])
@add.overload([list, Vec2])
def add(u, v):
    x, y = u
    a, b = v
    return Vec2(x + a, y + b)


@sub.overload([Vec2, tuple])
@sub.overload([Vec2, list])
@sub.overload([tuple, Vec2])
@sub.overload([list, Vec2])
def sub(u, v):
    x, y = u
    a, b = v
    return Vec2(x - a, y - b)


Vec3 = Vec[3, float]
Vec4 = Vec[4, float]
Direction2 = Direction[2, float]
Direction3 = Direction[3, float]
Direction4 = Direction[4, float]

#
# Direções unitárias e vetores nulos
#
ux2D = Direction2(1, 0)
uy2D = Direction2(0, 1)
ux3D = Direction3(1, 0, 0)
uy3D = Direction3(0, 1, 0)
uz3D = Direction3(0, 0, 1)
ux4D = Direction4(1, 0, 0, 0)
uy4D = Direction4(0, 1, 0, 0)
uz4D = Direction4(0, 0, 1, 0)
uw4D = Direction4(0, 0, 0, 1)
null2D = Vec2(0, 0)
null3D = Vec3(0, 0, 0)
null4D = Vec4(0, 0, 0, 0)

#
# Criação de vetores e pontos
#


def Vec(*args):
    '''Converte o argumento em um vetor de compontentes do tipo float'''

    if len(args) == 1:
        return Vec(*args[0])
    elif len(args) == 2:
        return Vec2.from_flat(args)
    elif len(args) == 3:
        return Vec3.from_flat(args)
    elif len(args) == 4:
        return Vec4.from_flat(args)
    else:
        return Vec.from_flat(args, dtype=float)


def asvector(u):
    if isinstance(u, (Vec2, Vec3, Vec4)):
        return u
    else:
        return Vec(*u)


def point(*args):
    '''Converte o argumento em um ponto de compontentes do tipo float'''

    if len(args) == 1:
        return point(*args[0])
    elif len(args) == 2:
        return Point2.from_flat(args)
    elif len(args) == 3:
        return Point3.from_flat(args)
    elif len(args) == 4:
        return Point4.from_flat(args)
    else:
        return Point.from_flat(args, dtype=float)


def direction(*args):
    '''Converte o argumento em um vetor direção unitária de compontentes do
    tipo float'''

    if len(args) == 1:
        return point(*args[0])
    elif len(args) == 2:
        return Direction2.from_flat(args)
    elif len(args) == 3:
        return Direction3.from_flat(args)
    elif len(args) == 4:
        return Direction4.from_flat(args)
    else:
        return Direction.from_flat(args, dtype=float)


#
# 2D projection functions
#
def shadow_x(A, B):
    '''Overlap between shapes A and B in the x direction'''

    return min(A.xmax, B.xmax) - max(A.xmin, B.xmin)


def shadow_y(A, B):
    '''Retorna o tamanho da sombra comum entre os objetos A e B no eixo y.
    Caso não haja superposição, retorna um valor negativo que corresponde ao
    tamanho do buraco'''

    return min(A.ymax, B.ymax) - max(A.ymin, B.ymin)


if __name__ == '__main__':
    print(Vec(1, 2) + (1, 2))
    print((1, 2) + Vec(1, 2))
    print(2 * Vec(1, 2))
    print(Vec(1, 2) * 2)
    print(asvector((1, 2)))



# -*- coding: utf8 -*-
'''
Ilustra a interseção entre duas figuras geométricas
'''

from FGAme import *
from FGAme.mathtools import clip, convex_hull

world = World()
A = Rectangle(shape=(200, 100), mass='inf')
B = RegularPoly(3, 200, pos=(70, 50), color='blue', mass='inf')
D = Poly(clip(A.vertices, B.vertices), color='red', mass='inf')
C = Circle(5, color='red')
vertices = A.vertices + B.vertices + D.vertices
conv = convex_hull(vertices)
E = Poly(conv, mass='inf', color='green')

world.add(E, layer=-1)
world.add(A)
world.add(B)
world.add(C, layer=1)
world.add(D)

for v in vertices:
    c = Circle(4, color='blue', pos=v)
    world.add(c, layer=2)

for v in conv:
    c = Circle(2, color='black', pos=v)
    world.add(c, layer=2)


world.run()



#-*- coding: utf8 -*-
'''
Implementa o exemplo de "pseudo-gravidade" da documentação.
'''

from FGAme import *
from FGAme.physics.forces import SpringTensorPair, GravityPair, SpringTensor, Gravity

# get_mainloop(fps=240)


class Gravity(World):

    def __init__(self):
        # Chamamos o __init__ da classe pai
        super(Gravity, self).__init__()

        # Criamos dois objetos
        A = Circle(20, pos=(500, 300), vel=(100, 300), color='red')
        B = Circle(20, pos=(400, 300), vel=(-100, -300))
        self.A, self.B = A, B
        self.add([A, B])

        # Definimos a força de interação entre ambos
        K = self.K = A.mass
        F = SpringTensorPair(A, B, (K, 2 * K))
        A.force, B.force = F.forces()

        E0 = F.energyT()
        #E0 = Fa.energyT() + Fb.energyT()

        @self.listen('frame-enter')
        def printP():
            print('%.e' % (F.energyT() + F.energyT() - E0))
            #print(F.energyT() / E0)

if __name__ == '__main__':
    Gravity().run()



'''
Gerencia os recursos disponíveis para o jogo na forma de arquivos.

Por padrão, a FGAme assume que os arquivos estão numa pasta chamada
``resources`` junto ao módulo principal de jogo.

'''

import os
import sys
from FGAme import draw

IMG_PRIORITIES = ['.png', '.gif', '.bmp', '.jpeg', '.jpg', '.tiff']
SND_PRIORITIES = ['.wav', '.ogg', '.mp3']
MEDIA_PRIORITIES = IMG_PRIORITIES + SND_PRIORITIES


class ResourceManager(object):

    def __init__(self, root=None):
        self._root = root

    @property
    def root(self):
        if self._root is not None:
            return self._root

        root = self.__find_root()
        if isinstance(root, str):
            root = os.path.expanduser(root)
            root = os.path.expandvars(root)
        self._root = root
        return root

    def get_root(self):
        '''Retorna o valor de root. Produz um RuntimeError caso root não tenha
        sido encontrado
        '''

        root = self.root
        if root is None:
            raise RuntimeError('could not find resource root')
        return root

    def __find_root(self):
        # Tenta encontrar o root em um dos caminhos especificados

        return self.__try_find_root_methods([
            self.__find_root_from_environ,
            self.__find_root_from_main,
        ])

    def __find_root_from_path(self, path):
        # Inspeciona caminho dado e tenta encontrar a pasta resources a partir
        # deste caminho. Caso não encontre, retorne None

        base = 'resources'
        while base:
            resources = os.path.join(path, 'resources')
            if os.path.exists(resources):
                return resources

            path, base = os.path.split(path)

    def __find_root_from_main(self):
        # Procura root a partir do pacote '__main__'

        main = sys.modules['__main__']
        path = os.path.split(main.__file__)[0]
        path = self.__find_root_from_path(path)
        if path is not None:
            return path

    def __find_root_from_environ(self):
        # Procura na variável de estado RESOURCES_PATH

        return os.environ.get('RESOURCES_PATH')

    def __try_find_root_methods(self, funcs):
        # Retorna o primeiro valor bem sucedido na lista de funções que
        # tentam encontrar a pasta resources

        for func in funcs:
            out = func()
            if out is not None:
                return out

    def find_data(self, name, mode='r', encoding=None, priorities=None):
        '''Retorna um objeto do tipo arquivo com o primeiro arquivo encontrado
        compatível com o nome dade.'''

        path = self.find_path(name, priorities)
        return open(self.get_data(path), mode, encoding=encoding)

    def find_path(self, name, priorities=None):
        '''Retorna o caminho para o arquivo de maior prioridade com o nome
        compatível com o valor fornecido
        '''

        root = self.get_root()
        root_size = len(root) + 1
        for ext in priorities:
            path = os.path.join(root, name + ext)
            if os.path.exists(path):
                return path[root_size:]
        raise RuntimeError('could not find %s' % os.path.join(root, name))

    def find_image(self, name):
        '''Retorna a imagem de maior prioridade com o nome compatível com o
        valor fornecido
        '''

        path = self.find_path(name, priorities=IMG_PRIORITIES)
        abspath = os.path.join(self.root, path)
        return draw.get_texture(abspath)

    def find_animation(self, name):
        '''Retorna a animação caminho para o arquivo de maior prioridade com o
        nome compatível com o valor fornecido
        '''

        return self.find_path(name, priorities=IMG_PRIORITIES)

    def find_abspath(self, path, priorities=None):
        '''Retorna o caminho absoluto de "path" no sistema de arquivos'''

        path = self.find_path(path, priorities)
        return os.path.join(self.root, path)


resources = ResourceManager()



# -*- coding: utf8 -*-
from __future__ import division
from FGAme import *

#conf.DEBUG = True

# Cria mundo e objetos
world = World(restitution=1)
world.add_bounds(width=10, use_poly=True)
obj1 = RegularPoly(N=3, length=130, pos=(200, 300),
                   vel=(200, 200), color='random', omega=2.2)
obj2 = RegularPoly(N=5, length=100, pos=(200, 450), theta=pi / 4,
                   color='random')
obj3 = RegularPoly(N=3, length=100, pos=(600, 300), theta=pi / 4,
                   color='random', mass='inf')

# Insere objetos e inicia rastreio de energia
world.register_energy_tracker()
world.add([obj1, obj2, obj3])
world.run()



from FGAme.events.base import EventDispatcher
from FGAme.events.util import pyname, signal
from FGAme.events.signals import DelegateSignal


class GlobalEventDispatcher(EventDispatcher):

    '''
    Classe cria uma única instância em FGAme.dispatcher que controla os eventos
    globais da FGAme.
    '''
    __instance = None

    def __new__(cls):
        if cls.__instance is None:
            new = object.__new__(cls)
            new.__init__()
            new._delegate_idx = 0
            new._delegates = {}
            return new
        else:
            return cls.__instance

    def new_signal(self, name, *filters, **kwds):
        '''Cria um novo sinal para o evento global.

        Possui uma assinatura variável onde o tipo de sinal é escolhido
        automaticamente a partir dos argumentos

        signal('foo', num_args=1)
            Sinal simples chamado 'foo' com apenas 1 argumento.

        signal('foo', 'bar')
            Sinal filtrado chamado 'foo' que requer um argumento extra 'bar'

        signal('foo', delegate_to=obj)
            Delega sinal 'foo' para o objeto fornecido em *delegate_to*
        '''

        cls = type(self)
        attr = pyname(name)

        # Organiza delegações e salva delegado em atributo
        if 'delegate_to' in kwds:
            delegate_obj = kwds['delegate_to']
            try:
                delegate_attr = self._delegates[id(delegate_obj)]
            except KeyError:
                delegate_attr = '_delegate_%s' % self._delegate_idx
                self._delegates[id(delegate_obj)] = delegate_attr
                self._delegate_idx += 1
            setattr(self, delegate_attr, delegate_obj)
            kwds['delegate_to'] = delegate_attr

        # Cria sinal
        signal_obj = signal(name, *filters, **kwds)
        if name in cls.__signals__:
            raise ValueError('global signal already exists: %r' % name)
        cls.__signals__[attr] = signal_obj
        setattr(cls, pyname(name), signal_obj)

        # Cria os métodos do tipo listen_<> e tipo trigger_<>
        lname = 'listen_' + attr
        tname = 'trigger_' + attr
        listen_method = signal_obj._factory_listen_method()
        trigger_method = signal_obj._factory_trigger_method()
        setattr(cls, lname, listen_method)
        setattr(cls, tname, trigger_method)

        # Inicializa SignalCtl
        ctl = signal_obj.default_control()
        setattr(self, '_%s_ctl' % attr, ctl)
        if not isinstance(signal, DelegateSignal):
            setattr(self, '_%s_book' % attr, ctl._runner)


dispatcher = GlobalEventDispatcher()



'''
Created on 18/06/2015

@author: chips
'''
from FGAme import *
conf.set_backend('pygame')
import pygame
import smallshapes as shapes
from random import random

world = World()
Circle(20, (500, 500), world=world)

# Cria a caixa e a imagem associada
imgs = [pygame.image.load('resources/alien%s.gif' % i) for i in range(1, 4)]
_x, _y, dx, dy = imgs[0].get_rect()
box = AABB(0, dx, 0, dy, color=None, world=world)

# Move o macaco de acordo com as setas
world.listen('long-press', 'left')(lambda: box.move((-2, 0)))
world.listen('long-press', 'right')(lambda: box.move((4, 0)))
world.listen('long-press', 'up')(lambda: box.move((0, 2)))
world.listen('long-press', 'down')(lambda: box.move((0, -2)))
idx = 0


@world.listen('pre-draw')
def draw_sprite(screen):
    '''Recebe um objeto do tipo Screen e realiza desenhos na tela'''

    global idx
    idx += 0.1

    dx, dy = box.shape
    screen.draw_circle(shapes.Circle(40, Vec2(400, 300)))

    # Recupera o Surface para utilizar manualmente as funções do pygame
    pg_screen = screen.get_pygame_screen()
    pg_screen.blit(imgs[int(idx % 3)], (box.xmin, 600 - box.ymax, dx, dy))


@world.listen('key-down', 'space')
def shoot():
    world.add(Circle(3, vel=(200, 200 * random() - 100), pos=box.pos_right))

world.run()



# -*- coding: utf8 -*-
import six
from FGAme.events.util import pyname
from FGAme.events.signals import Signal, DelegateSignal


class EventDispatcherMeta(type):

    '''Meta-classe para EventDispatcher.

    Fabrica e registra automaticamente os métodos listen_* e trigger_*.
    Atualiza os __slots__ caso a classe filho os utilize. Finalmente salva
    algumas constantes úteis para a introspecção:

        __signals__
            Dicionário mapeando todos os nomes de sinais com os seus
            respectivos objetos.
        __marked_listen__
            Dicionário com todos os métodos que foram marcados com o decorador
            @listen durante a definição da classe.
    '''

    def __new__(cls, name, bases, ns):
        ns = cls._populate_namespace(name, bases, ns)
        return type.__new__(cls, name, bases, ns)

    @classmethod
    def _populate_namespace(cls, name, bases, ns):
        '''Retorna o namespace ns acrescentado dos métodos trigger_* e
        listen_* apropriados para a classe em questão'''

        ns = dict(ns)

        # Lê os sinais e monta um dicionário de sinais
        signals = {}
        for C in reversed(bases):
            signals.update(getattr(C, '__signals__', {}))
        for attr, value in ns.items():
            if isinstance(value, Signal):
                signals[attr] = value
        ns['__signals__'] = signals

        # Cria os métodos do tipo listen_* e tipo trigger_*
        for signal in signals.values():
            lname = 'listen_' + pyname(signal.name)
            tname = 'trigger_' + pyname(signal.name)
            ns['_' + lname] = signal._factory_listen_method()
            if lname not in ns:
                ns[lname] = ns['_' + lname]
            ns['_' + tname] = signal._factory_trigger_method()
            if tname not in ns:
                ns[tname] = ns['_' + tname]

        # Escaneia todos os métodos decorados com @listen
        listen = {}
        for C in reversed(bases):
            listen.update(getattr(C, '__marked_listen__', {}))
        for attr, value in ns.items():
            if hasattr(value, '_listen_args'):
                for name, args, kwds in getattr(value, '_listen_args'):
                    L = listen.setdefault(name, [])
                    L.append((attr, args, kwds))

        ns['__marked_listen__'] = listen

        # Atribui valores de slots, caso necessário
        if '__slots__' in ns:
            slots = list(ns['__slots__'])
            for signal_name, signal in signals.items():
                if not isinstance(signal, DelegateSignal):
                    slots.append('_%s_ctl' % signal_name)
                    slots.append('_%s_book' % signal_name)
            ns['__slots__'] = slots

        return ns

    @classmethod
    def decorate(cls, ev_type):
        '''Decora uma classe que não inclui EventDispatcher na hierarquia e
        insere métodos/propriedades etc para que ela se comporte como um
        membro de EventDispatcher.

        Esta função pode ser útil quando EventDispatcher não pode ser inserida
        na hierarquia de classes por um conflito de metaclasses ou por um
        conflito com o campo __slots__ que previne herança múltipla.

        O método __init__ é renomeado para _init_events para poder se tornar
        acessível aos membros da classe decorada.

        Observe que classes decoradas desta maneira não são subclasses de
        EventDispatcher. Deste modo, os mecanismos de criação e registro de
        novos sinais não funcionarão automaticamente. Caso uma sub-classe
        queira definir um novo sinal, será necessário decorá-la manualmente.
        '''

        ev_type._init_events = EventDispatcher.__init__
        ev_type.listen = EventDispatcher.listen
        ev_type.trigger = EventDispatcher.trigger

        # TODO: implementar @EventDispatcherMeta.decorate
        name = ev_type.__name__
        bases = ev_type.__bases__
        ns = dict(ev_type.__dict__)
        ns = cls._populate_namespace(name, bases, ns)

        # Remove read-only attributes
        blacklist = ['__doc__', '__dict__', '__weakref__']
        for k in blacklist:
            ns.pop(k, None)

        # Write new attributes
        for k, new in ns.items():
            old = getattr(ev_type, k, None)
            if old is not new:
                try:
                    setattr(ev_type, k, new)
                except AttributeError:
                    tname = ev_type.__name__
                    msg = 'could not write %s=%r for type %s' % (k, new, tname)
                    raise ValueError(msg)
        return ev_type


@six.add_metaclass(EventDispatcherMeta)
class EventDispatcher(object):

    '''Implementa o despachante de eventos da biblioteca FGAme.'''

    __slots__ = []

    def __init__(self):
        for signal in self.__signals__.values():
            ctl = signal.default_control(self)
            setattr(self, '_%s_ctl' % pyname(signal.name), ctl)
            if not isinstance(signal, DelegateSignal):
                setattr(self, '_%s_book' % pyname(signal.name), ctl._runner)

            # Registra os métodos criados pelo mecanismo on_signal...
            auto_method = getattr(self, 'on_' + pyname(signal.name), None)
            if auto_method is not None:
                kwargs = {'owner': self}
                if signal._num_filters == 0:
                    self.listen(signal.name, auto_method, **kwargs)
                else:
                    args = (None,) * signal._num_filters
                    args += (auto_method,)
                    self.listen(signal.name, *args, **kwargs)

            # Registra os métodos criados pelo mecanismo @listen
            marked_listen = self.__marked_listen__.get(signal.name, [])
            for (func_name, args, kwargs) in marked_listen:
                handler = getattr(self, func_name)
                filters = (
                    args[:signal._num_filters]
                    or (None, ) * signal._num_filters
                )
                args = args[signal._num_filters:]
                args = filters + (handler,) + args
                kwargs = kwargs.copy()
                kwargs['owner'] = self
                self.listen(signal.name, *args, **kwargs)

    def trigger(self, signal, *args, **kwds):
        '''Aciona o sinal com os argumentos fornecidos.'''

        implementation = getattr(self, 'trigger_' + pyname(signal))
        return implementation(*args, **kwds)

    def listen(self, signal, *args, **kwds):
        '''Registra um handler para um determinado sinal.

        A assinatura correta para esta função depende do tipo de sinal
        considerado. Para sinais simples, basta utilizar::

            obj.listen(<nome do sinal>, <handler>)

        Sinais filtrados (aqueles que pedem um ou mais argumentos adicionais),
        devem ser registrados como::

            obj.listen(<nome do sinal>, <filtro>, <handler>)

        Opcionalmente, é possível acrescentar argumentos por nome ou posição
        que serão passados automaticamente para o handler quando o sinal for
        acionado.
        '''

        implementation = getattr(self, 'listen_' + pyname(signal))
        return implementation(*args, **kwds)

    def destroy_handlers(self, signal=None):
        '''Destroy todos os handlers associados ao sinal dado.

        Retorna um dicionário que mapeia o nome dos sinais à lista de handlers
        removidos.'''

        if signal is None:
            out = {}
            for signal in self.__signals__:
                handlers = self.destroy_handlers(signal)
                out.update(handlers)
            return out
        else:
            signalctl = getattr(self, signal)
            handlers = signalctl.remove(owner=self)
            return {signal: handlers}



# -*- coding: utf8 -*-_

'''
Este exemplo demonstra a resposta a colisões com atrito utilizando duas caixas
AABB.
'''

from FGAme import World, AABB, Circle, Rectangle

# Cria mundo com coeficiente de atrito global não-nulo
world = World()
world.add_bounds(width=10)

stacks = []
stacks.append('aabbs')
stacks.append('circles')
stacks.append('rects')


# Cria pilha de AABBs
H = 550
dH = 200
if 'aabbs' in stacks:
    world.add(AABB(pos=(50, H), shape=(40, 40), color='red', vel=(200, 0)))
    for i in range(10):
        world.add(AABB(pos=(150 + i * 50, H), shape=(40, 40)))
    world.add(AABB(pos=(150 + 10 * 50, H), shape=(40, 40), color='red'))
    world.add(AABB(pos=(50, H - 50),
                   shape=(40, 40), color='red', vel=(200, 0)))
    H -= dH

# Cria pilha de círculos
if 'circles' in stacks:
    world.add(Circle(20, pos=(50, H), color='red', vel=(200, 0)))
    for i in range(10):
        world.add(Circle(20, pos=(150 + i * 50, H)))
    world.add(Circle(20, pos=(150 + 10 * 50, H), color='red'))
    world.add(Circle(20, pos=(50, H - 50), color='red', vel=(200, 0)))
    H -= dH

# Cria pilha de retangulos
if 'rects' in stacks:
    world.add(Rectangle(pos=(50, H), shape=(40, 40), color='red',
                        vel=(200, 0)))
    for i in range(10):
        world.add(Rectangle(pos=(150 + i * 50, H), shape=(40, 40)))
    world.add(Rectangle(pos=(150 + 10 * 50, H), shape=(40, 40), color='red'))
    world.add(Rectangle(pos=(50, H - 50),
                        shape=(40, 40), color='red', vel=(200, 0)))


#world.add(Circle(10, pos=(30, 30), vel=(300, 400)))
#world.add(Circle(10, pos=(30, 30), vel=(400, 300)))

# Registra energias
world.register_energy_tracker()

# Inicia a simulação
world.run()



# -*- coding: utf8 -*-

from collections import MutableSequence
from FGAme.mathtools import shadow_y
from FGAme.physics import CBBContact, AABBContact
from FGAme.physics import get_collision
from FGAme.physics.flags import BodyFlags


class AbstractCollisionPhase(MutableSequence):

    '''Base para BroadPhase e NarrowPhase'''

    __slots__ = ['world', '_data']

    def __init__(self, data=[], world=None):
        self.world = world
        self._data = []
        self._data.extend(data)

    def __call__(self, objects):
        self.update(objects)
        return self

    def __repr__(self):
        tname = type(self).__name__
        return '%s(%r)' % (tname, self._data)

    def update(self, objects):
        '''Atualiza a lista de pares utilizando a lista de objetos dada.'''

        raise NotImplementedError

    def objects(self):
        '''Iterador sobre a lista com todos os objetos obtidos na fase de
        colisão'''

        objs = set()
        for A, B in self._data:
            objs.add(A)
            objs.add(B)
        return iter(objs)

    # MutableSequence interface ###############################################
    def __len__(self):
        return len(self._data)

    def __iter__(self):
        for p in self._data:
            yield p

    def __getitem__(self, i):
        return self._data[i]

    def __setitem__(self, i, value):
        self._data[i] = value

    def __delitem__(self, i, value):
        del self._data[i]

    def pop(self, idx=None):
        if idx is None:
            return self._data.pop()
        else:
            return self._data.pop(idx)

    def insert(self, idx, value):
        self._data.insert(idx, value)

    def append(self, value):
        self._data.append(value)

    def remove(self, value):
        self._data.remove(value)

    def sort(self, *args, **kwds):
        self._data.sort(*args, **kwds)

###############################################################################
#                               Broad phase
###############################################################################


class BroadPhase(AbstractCollisionPhase):

    '''Controla a broad-phase do loop de detecção de colisões de uma
    simulação.

    Um objeto do tipo BroadPhase possui uma interface simples que define dois
    métodos:

        bf.update(L) -> executa algoritmo em lista de objetos L
        iter(bf)     -> itera sobre todos os pares gerados no passo anterior

    '''

    __slots__ = []

    def pairs(self):
        '''Retorna a lista de pares encontradas por update'''

        return list(self._data)


class BroadPhaseAABB(BroadPhase):

    '''Implementa a broad-phase detectando todos os pares de AABBs que estão
    em contato no frame'''

    __slots__ = []

    def update(self, L):
        IS_SLEEP = BodyFlags.is_sleeping
        can_collide = self.world.can_collide
        col_idx = 0
        objects = sorted(L, key=lambda obj: obj.xmin)
        self._data[:] = []

        # Os objetos estão ordenados. Este loop detecta as colisões da CBB e
        # salva o resultado na lista broad collisions
        for i, A in enumerate(objects):
            A_right = A.xmax
            A_dynamic = A.is_dynamic()

            for j in range(i + 1, len(objects)):
                B = objects[j]
                if not can_collide(A, B):
                    continue

                # Procura na lista enquanto xmin de B for menor que xmax de A
                B_left = B.xmin
                if B_left > A_right:
                    break

                # Não detecta colisão entre dois objetos estáticos/cinemáticos
                if not A_dynamic and not B.is_dynamic():
                    continue
                if A.flags & B.flags & IS_SLEEP:
                    continue

                # Testa a colisão entre as AABBs
                if shadow_y(A, B) <= 0:
                    continue

                # Adiciona à lista de colisões grosseiras
                col_idx += 1
                self._data.append(AABBContact(A, B))


class BroadPhaseCBB(BroadPhase):

    '''Implementa a broad-phase detectando todos os pares de CBBs que estão
    em contato no frame'''

    __slots__ = []

    def update(self, L):
        can_collide = self.world.can_collide
        L = sorted(L, key=lambda obj: obj.pos.x - obj.cbb_radius)
        N = len(L)
        self._data[:] = []

        # Os objetos estão ordenados. Este loop detecta as colisões da CBB e
        # salva o resultado na lista broad collisions
        for i, A in enumerate(L):
            rA = A.cbb_radius
            Amax = A.pos.x + rA

            for j in range(i + 1, N):
                B = L[j]
                if not can_collide(A, B):
                    continue
                rB = B.cbb_radius

                # Procura na lista enquanto xmin de B for menor que xmax de A
                if B._pos.x - rB > Amax:
                    break

                # Testa a colisão entre os círculos de contorno
                if (A.pos - B.pos).norm() > rA + rB:
                    continue

                # Adiciona à lista de colisões grosseiras
                self._data.append(CBBContact(A, B))


class BroadPhaseMixed(BroadPhase):

    '''Implementa a broad-phase detectando todos os pares de CBBs que estão
    em contato no frame'''

    __slots__ = []

    def update(self, L):
        IS_SLEEP = BodyFlags.is_sleeping
        can_collide = self.world.can_collide
        col_idx = 0
        objects = sorted(L, key=lambda obj: obj.pos.x - obj.cbb_radius)
        self._data[:] = []

        # Os objetos estão ordenados. Este loop detecta as colisões da CBB e
        # salva o resultado na lista broad collisions
        for i, A in enumerate(objects):
            A_radius = A.cbb_radius
            A_right = A.pos.x + A_radius
            A_dynamic = A.is_dynamic()

            for j in range(i + 1, len(objects)):
                B = objects[j]
                if not can_collide(A, B):
                    continue

                B_radius = B.cbb_radius

                # Procura na lista enquanto xmin de B for menor que xmax de A
                B_left = B._pos.x - B_radius
                if B_left > A_right:
                    break

                # Não detecta colisão entre dois objetos estáticos/cinemáticos
                if not A_dynamic and not B.is_dynamic():
                    continue
                if A.flags & B.flags & IS_SLEEP:
                    continue

                # Testa a colisão entre os círculos de contorno
                if (A.pos - B.pos).norm() > A_radius + B_radius:
                    continue

                # Adiciona à lista de colisões grosseiras
                col_idx += 1
                if has_overlap(A.aabb, B.aabb):
                    self._data.append(AABBContact(A, B))


###############################################################################
#                               Narrow phase
###############################################################################
class NarrowPhase(AbstractCollisionPhase):

    '''Implementa a fase fina da detecção de colisão'''

    __slots__ = []

    def update(self, broad_cols):
        '''Escaneia a lista de colisões grosseiras e detecta quais delas
        realmente aconteceram'''

        # Detecta colisões e atualiza as listas internas de colisões de
        # cada objeto
        self._data = cols = []

        for A, B in broad_cols:
            if A._invmass > B._invmass:
                A, B = B, A
            col = get_collision(A, B)

            if col is not None:
                A.add_contact(col)
                B.add_contact(col)
                col.world = self.world
                cols.append(col)

    def get_groups(self, cols=None):
        '''Retorna uma lista com todos os grupos de colisões fechados'''

        if cols is None:
            cols = self

        meta_cols = BroadPhaseAABB(cols)
        print(meta_cols)


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# -*- coding: utf8 -*-
from contextlib import contextmanager
from FGAme.mathtools import Vec2, shapes
from FGAme.draw import color_property, Color

black = Color('black')
white = Color('white')


###############################################################################
#                          Classe Screen genérica
###############################################################################
# TODO: refatorar para ter uma classe de Canvas separada da classe Window?
class Screen(object):

    '''Classe que define a funcionalidade básica de todos os backends que
    gerenciam a disponibilização de imagens na tela do computador.

    Existem dois modelos de renderização disponíveis

        * O modelo Canvas (ou tela) utiliza a metáfora de pintura, onde os
          pixels da tela são "pintados" a cada frame de renderização.

        * O modelo LiveTree delega a pintura para um backend mais básico que
          determina o instante preciso do frame de renderização e quais partes
          da tela devem ser re-escritas com base numa árvore que guarda e
          atualiza as funções de renderização.
    '''
    __instance = None
    is_canvas = False
    background = color_property('background')

    def __new__(cls, *args, **kwds):
        if cls.__instance is not None:
            raise TypeError('cannot create two instances of singleton object')
        return object.__new__(cls)

    def __init__(self, shape=(800, 600), pos=(0, 0), zoom=1, background=None):
        self.width, self.height = shape
        self.pos = Vec2(*pos)
        self.zoom = zoom
        self.background = background
        self._direct = True
        self.visible = False

    def init(self):
        '''Deve ser chamado como primeira função para iniciar explicitamente a
        tela e para abrir e mostrar a janela de jogo.'''

        pass

    def show(self):
        '''Deve ser chamado como primeira função para iniciar explicitamente a
        tela e para abrir e mostrar a janela de jogo.'''

        self.visible = True

    @property
    def shape(self):
        return self.width, self.height


###############################################################################
#     Classe Canvas: backends baseados em renderização do tipo "pintura"
###############################################################################
class Canvas(Screen):

    '''Sub-classes implementam a metáfora de "pintura" para a renderização das
    imagens.

    As sub-implementações devem saber como renderizar objetos geométricos
    básicos como círculos, linhas, pontos, polígonos, etc.
    '''
    is_canvas = True

    def __init__(self, shape=(800, 600), pos=(0, 0), zoom=1, background=None):
        super(Canvas, self).__init__(shape, pos, zoom, background)
        self._drawing_funcs = {}

    def show(self):
        if not self.visible:
            super(Canvas, self).show()
            self.clear_background('white')
            self.flip()

    def flip(self):
        '''Transmite o buffer de pintura para a tela do computador'''

        raise NotImplementedError

    # Context managers ########################################################
    @contextmanager
    def autoflip(self):
        '''Ao sair do bloco `with`, executa automaticamente o método flip()'''

        try:
            yield None
        finally:
            self.flip()

    @contextmanager
    def painting(self):
        '''Semelhante ao método `autoflip()`, mas limpa a tela antes de fazer
        os desenhos.

        Deste modo, somente as imagens referentes aos comandos dentro do bloco
        serão exibidas.'''

        self.clear_background(self.background or Color('white'))
        try:
            yield None
        finally:
            self.flip()

    # Objetos primitivos ######################################################
    # Estas funções desenham objetos primitivos na tela sem se atentar para
    # transformações de escala, translação e rotação. São as operações
    # primitivas que devem ser sobrescritas por cada backend suportado.
    # A maior parte das implementações padrão é vazia

    # Entes primitivos
    def draw_raw_pixel(self, pos, color=black):
        '''Desenha um pixel na posição dada'''
        raise NotImplementedError

    def draw_raw_segment(self, segment, width=1.0, color=black):
        '''Desenha um segmento de reta'''
        raise NotImplementedError

    def draw_raw_line(self, line, width=1.0, color=black):
        '''Desenha uma linha infinita'''
        # TODO: implementar a partir de raw_segment()
        raise NotImplementedError

    def draw_raw_ray(self, line, width=1.0, color=black):
        '''Desenha um raio (ou linha semi-infinita)'''
        # TODO: implementar a partir de raw_segment()
        raise NotImplementedError

    # Figuras sólidas
    def draw_raw_circle_solid(self, circle, color=black):
        '''Desenha um círculo sólido na tela'''
        raise NotImplementedError

    def draw_raw_circle_border(self, circle, width=1.0, color=black):
        '''Desenha a borda de um círculo'''
        raise NotImplementedError

    def draw_raw_aabb_solid(self, poly, color=black):
        '''Desenha uma aabb sólida'''
        raise NotImplementedError

    def draw_raw_aabb_border(self, aabb, width=1.0, color=black):
        '''Desenha a borda de uma aabb'''
        raise NotImplementedError

    def draw_raw_poly_solid(self, poly, color=black):
        '''Desenha um polígono sólido'''
        raise NotImplementedError

    def draw_raw_poly_border(self, poly, width=1.0, color=black):
        '''Desenha a borda de um polígono'''
        # TODO: implementar a partir de raw_segment()
        raise NotImplementedError

    def draw_raw_texture(self, texture, start_pos=(0, 0)):
        '''Desenha uma textura na tela'''
        raise NotImplementedError

    def draw_raw_image(self, image):
        '''Desenha uma imagem na tela'''

        self.draw_raw_texture(image.texture, image.pos_sw)

    def clear_background(self, color):
        '''Limpa o fundo com a cor especificada'''

        raise NotImplementedError

    # Objetos derivados #######################################################
    # Estas são as funções que devem ser utilizadas diretamente pelos usuários
    # da FGAme. Elas desenham um objeto a partir de uma figura primitiva e
    # aplicam automaticamente as transformações de escala, translação e rotação
    # necessárias.
    def draw_circle(self, circle, solid=None, color=None,
                    line_width=None, line_color=None):
        '''Desenha um círculo na tela.'''

        if not self._direct:
            raise RuntimeError

        if solid:
            self.draw_raw_circle_solid(circle, color)
        if line_color is not None and line_width:
            self.draw_raw_circle_border(circle, line_width, line_color)

    def draw_aabb(self, aabb, solid=True, color=black,
                  line_width=0.0, line_color=black):
        '''Desenha uma AABB na tela.'''

        if not self._direct:
            raise RuntimeError

        if solid:
            self.draw_raw_aabb_solid(aabb, color)
        if line_color is not None and line_width:
            self.draw_raw_aabb_border(aabb, line_width, line_color)

    def draw_poly(self, poly, solid=True, color=black,
                  line_width=0.0, line_color=black):
        '''Desenha um polígono na tela.'''

        if not self._direct:
            raise RuntimeError

        if solid:
            self.draw_raw_poly_solid(poly, color)
        if line_color is not None and line_width:
            self.draw_raw_poly_border(poly, line_width, line_color)

    def draw_segment(self, segment, width=0.0, color=black):
        '''Desenha um segmento de reta.'''

        if not self._direct:
            raise RuntimeError

        self.draw_raw_segment(segment, width, color)

    def draw_ray(self, ray, width=0.0, color=black):
        '''Desenha um raio (reta semi-finita)'''

        raise NotImplementedError

    def draw_line(self, ray, width=0.0, color=black):
        '''Desenha uma reta infinita'''

        raise NotImplementedError

    def draw_path(self, path, width=0.0, color=black):
        '''Desenha um caminho formado por vários pontos.'''

        if not self._direct:
            raise RuntimeError

        points = iter(path)
        pt0 = next(points)
        for pt1 in points:
            self.draw_raw_segment(shapes.Segment(pt0, pt1), width, color)
            pt0 = pt1

    def draw_image(self, image):
        if self._direct:
            self.draw_raw_image(image)
        else:
            raise NotImplementedError

    # Interface de objetos do tipo Drawable. Objetos podem sinalizar que são
    # desenhaveis quando implementam o método obj.draw(canvas). O método draw
    # é chamado com um objeto do tipo Canvas como atributo e deve se desenhar
    # a partir de chamadas às funções primitivas de desenho.
    def draw(self, object):
        '''Desenha objetos do tipo Drawable.

        Objetos do tipo Drawable possuem uma função obj.draw(screen) que
        desenha o objeto na tela fornecida como parâmetro de entrada.
        '''
        object.draw(self)



# -*- coding: utf8 -*-

from FGAme.mathtools import Vec2, RotMat2, sin, pi
from FGAme.mathtools import area, aabb_bbox, center_of_mass, ROG_sqr, shapes
from . import Body

__all__ = ['Poly', 'RegularPoly', 'Rectangle']


class Poly(Body):

    '''Define um polígono arbitrário de N lados.'''

    __slots__ = ['_vertices', 'num_sides', '_normals_idxs', 'num_normals']

    def __init__(self,
                 vertices,
                 pos=None, vel=(0, 0), theta=0.0, omega=0.0,
                 mass=None, density=None, inertia=None, **kwds):

        vertices = [Vec2(*pt) for pt in vertices]
        pos_cm = center_of_mass(vertices)
        vertices = [v - pos_cm for v in vertices]
        self._vertices = vertices

        # Cache de vértices
        self._cache_theta = None
        self._cache_rvertices_last = None
        self._cache_rbbox_last = None
        self.cbb_radius = max(v.norm() for v in vertices)
        super(Poly, self).__init__(pos_cm, vel, theta, omega,
                                   mass=mass, density=density, inertia=inertia,
                                   **kwds)

        self.num_sides = len(vertices)
        self._normals_idxs = self.get_li_indexes()
        self.num_normals = len(self._normals_idxs or self.vertices)

        # Aceleramos um pouco o cálculo para o caso onde todas as normais são
        # LI. entre si. Isto é sinalizado por self._normals_idx = None, que
        # implica que todas as normais do polígono devem ser recalculadas a
        # cada frame
        if self.num_normals == self.num_sides:
            self._normals_idxs = None

        # Movemos para a posição especificada caso _pos seja fornecido
        if pos is not None:
            self._pos = Vec2(*pos)

    def get_li_indexes(self):
        '''Retorna os índices referents às normais linearmente independentes
        entre si.

        Este método é invocado apenas na inicialização do objeto e pode
        involver testes de independencia linear relativamente caros.
        '''

        normals = [self.get_normal(i).normalize()
                   for i in range(self.num_sides)]
        LI = []
        LI_idx = []
        for idx, n in enumerate(normals):
            for n_other in LI:
                # Produto vetorial nulo ==> dependência linear
                if abs(n.cross(n_other)) < 1e-3:
                    break
            else:
                # Executado se o loop "for" não terminar em um break
                # Implica em independência linear
                LI.append(n)
                LI_idx.append(idx)
        return LI_idx

    def get_side(self, i):
        '''Retorna um vetor na direção do i-ésimo lado do polígno. Cada
        segmento é definido pela diferença entre o (i+1)-ésimo ponto e o
        i-ésimo ponto.
        '''

        points = self.vertices
        return points[(i + 1) % self.num_sides] - points[i]

    def get_normal(self, i):
        '''Retorna a normal unitária associada ao i-ésimo segmento. Cada
        segmento é definido pela diferença entre o (i+1)-ésimo ponto e o
        i-ésimo ponto.'''

        points = self.vertices
        x, y = points[(i + 1) % self.num_sides] - points[i]
        return Vec2(y, -x).normalize()

    def get_normals(self):
        '''Retorna uma lista com as normais linearmente independentes.'''

        if self._normals_idxs is None:
            N = self.num_sides
            points = self.vertices
            segmentos = (points[(i + 1) % N] - points[i] for i in range(N))
            return [Vec2(y, -x).normalize() for (x, y) in segmentos]
        else:
            return [self.get_normal(i) for i in self._normals_idxs]

    def is_internal_point(self, pt):
        '''Retorna True se um ponto for interno ao polígono.'''

        n = self.get_normal
        P = self.vertices
        return all((pt - P[i]).dot(n(i)) <= 0 for i in range(self.num_sides))

    ###########################################################################
    #                     Sobrescrita de métodos
    ###########################################################################
    @property
    def bounding_box(self):
        return shapes.Poly(self.vertices)

    @property
    def vertices(self):
        pos = self._pos
        return [v + pos for v in self._rvertices]

    @property
    def _rvertices(self):
        if self._theta == self._cache_theta:
            return self._cache_rvertices_last
        else:
            R = RotMat2(self._theta)
            vert = [R * v for v in self._vertices]
            xmin = min(v.x for v in vert)
            xmax = max(v.x for v in vert)
            ymin = min(v.y for v in vert)
            ymax = max(v.y for v in vert)
            bbox = (xmin, xmax, ymin, ymax)

            self._cache_rvertices_last = vert
            self._cache_theta = self._theta
            self._cache_rbbox_last = bbox

            return vert

    @property
    def _rbbox(self):
        if self._theta == self._cache_theta:
            return self._cache_rbbox_last
        else:
            self._rvertices
            return self._cache_rbbox_last

    def scale(self, scale, update_physics=False):
        self._vertices = [scale * v for v in self._vertices]

    def area(self):
        return area(self._vertices)

    def ROG_sqr(self):
        return ROG_sqr(self._vertices)

    @property
    def xmin(self):
        return self._pos.x + self._rbbox[0]

    @property
    def xmax(self):
        return self._pos.x + self._rbbox[1]

    @property
    def ymin(self):
        return self._pos.y + self._rbbox[2]

    @property
    def ymax(self):
        return self._pos.y + + self._rbbox[3]

    @property
    def shape_bb(self):
        return shapes.Poly(self.vertices)

###############################################################################
#                         Especialização de polígonos
###############################################################################


class RegularPoly(Poly):

    __slots__ = ['length']

    def __init__(self, N, length,
                 pos=(0, 0), vel=(0, 0), theta=0.0, omega=0.0,
                 **kwds):
        '''Cria um polígono regoular com N lados de tamanho "length".'''

        self.length = length
        vertices = self._vertices(N, length, pos)
        super(RegularPoly, self).__init__(vertices, None, vel, theta, omega,
                                          **kwds)

    def _vertices(self, N, length, pos):
        self.length = length
        alpha = pi / N
        theta = 2 * alpha
        b = length / (2 * sin(alpha))
        P0 = Vec2(b, 0)
        pos = Vec2(*pos)
        return [(P0.rotate(n * theta)) + pos for n in range(N)]


class Rectangle(Poly):

    __slots__ = []

    def __init__(self, xmin=None, xmax=None, ymin=None, ymax=None,
                 pos=None, vel=(0, 0), theta=0.0, omega=0.0,
                 bbox=None, rect=None, shape=None, **kwds):
        '''Cria um retângulo especificando ou a caixa de contorno ou a posição
        do centro de massa e a forma.

        Pode ser inicializado como uma AABB, mas gera um polígono como
        resultado.

        >>> r = Rectangle(shape=(200, 100))
        >>> r.rotate(pi/4)
        >>> r.bbox                                         # doctest: +ELLIPSIS
        (-106.066..., 106.066..., -106.066..., 106.066...)

        '''

        xmin, xmax, ymin, ymax = aabb_bbox(
            xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax,
            bbox=bbox, rect=rect, shape=shape, pos=pos)

        super(Rectangle, self).__init__(
            [(xmax, ymin), (xmax, ymax), (xmin, ymax), (xmin, ymin)],
            None, vel, theta, omega, **kwds)


def triangle(cls, sides, pos=(0, 0), **kwds):
    '''Cria um triângulo especificando o tamanho dos lados'''
    pass


def blob(cls, N, scale, pos=(0, 0), **kwds):
    '''Cria um polígono convexo aleatório especificando o número de lados e
    um fator de escala.'''
    pass


if __name__ == '__main__':
    import doctest
    doctest.testmod()
